@@ -43,6 +43,7 @@ check_custom() {
 
 fail() {
     pkgname=$1
+		rm -f config.mk
 		echo ""
 		echo "You need package ${pkgname} in order to compile this program."
 		echo "Please make sure it is installed."
@@ -56,7 +57,7 @@ fail_custom() {
 	exit 1
 }
 
-echo -n "" > config.mk
+echo "" > config.mk
 
 check_pkg "sqlite3" || fail "sqlite3"
 check_pkg "libcurl" || check_custom "libcurl" "curl-config" || fail "libcurl"
