@@ -15,22 +15,23 @@ public:
 	int count;			// number of errors detected
 
 	Errors();
-	void SynErr(int line, int col, int n);
-	void Error(int line, int col, wchar_t *s);
-	void Warning(int line, int col, wchar_t *s);
-	void Warning(wchar_t *s);
-	void Exception(wchar_t *s);
+	void SynErr(int n);
+	void Error();
+	void Warning();
+	void Exception();
 
 }; // Errors
 
 class Parser {
 private:
-	int _EOF;
-	int _openblock;
-	int _closeblock;
-	int _ident;
-	int _stringliteral;
-	int _numliteral;
+	enum {
+		_EOF=0,
+		_openblock=1,
+		_closeblock=2,
+		_ident=3,
+		_stringliteral=4,
+		_numliteral=5,
+	};
 	int maxT;
 
 	Token *dummyToken;
