@@ -12,6 +12,7 @@ namespace newsbeuter {
 
 	struct config_action_handler {
 		virtual action_handler_status handle_action(const std::string& action, const std::vector<std::string>& params) = 0;
+		config_action_handler() { }
 		virtual ~config_action_handler() { }
 	};
 
@@ -27,6 +28,7 @@ namespace newsbeuter {
 			std::vector<std::vector<std::string> > parsed_content;
 			std::map<std::string,config_action_handler *> action_handlers;
 			std::set<std::string> included_files;
+			const char * get_errmsg(action_handler_status status);
 	};
 
 	class null_config_action_handler : public config_action_handler {
