@@ -24,11 +24,12 @@ namespace newsbeuter {
 
 			void reload_all(bool unattended = false);
 			void reload_indexes(const std::vector<int>& indexes, bool unattended = false);
+			void reload_range(unsigned int start, unsigned int end, unsigned int size, bool unattended = false);
 			void start_reload_all_thread(std::vector<int> * indexes = 0);
 
-			rss_feed * get_feed(unsigned int pos);
-			rss_feed * get_feed_by_url(const std::string& feedurl);
-			std::vector<rss_item> search_for_items(const std::string& query, const std::string& feedurl);
+			std::tr1::shared_ptr<rss_feed> get_feed(unsigned int pos);
+			std::tr1::shared_ptr<rss_feed> get_feed_by_url(const std::string& feedurl);
+			std::vector<std::tr1::shared_ptr<rss_item> > search_for_items(const std::string& query, const std::string& feedurl);
 			inline unsigned int get_feedcount() { return feeds.size(); }
 
 			inline void unlock_reload_mutex() { reload_mutex->unlock(); }
@@ -38,18 +39,18 @@ namespace newsbeuter {
 			void update_visible_feeds();
 			void mark_all_read(unsigned int pos);
 			void catchup_all();
-			inline void catchup_all(rss_feed& feed) { rsscache->catchup_all(feed); }
+			inline void catchup_all(std::tr1::shared_ptr<rss_feed> feed) { rsscache->catchup_all(feed); }
 			inline bool get_refresh_on_start() { return refresh_on_start; }
 			bool is_valid_podcast_type(const std::string& mimetype);
-			void enqueue_url(const std::string& url);
+			void enqueue_url(const std::string& url, std::tr1::shared_ptr<rss_feed> feed);
 			void notify(const std::string& msg);
 
 			void reload_urls_file();
+			void edit_urls_file();
 
-			inline std::vector<rss_feed>& get_all_feeds() { return feeds; }
+			inline std::vector<std::tr1::shared_ptr<rss_feed> >& get_all_feeds() { return feeds; }
 
-			// TODO: move somewhere else...
-			void set_feedptrs(rss_feed& feed);
+			void set_feedptrs(std::tr1::shared_ptr<rss_feed> feed);
 
 			inline filtercontainer& get_filters() { return filters; }
 
@@ -59,7 +60,7 @@ namespace newsbeuter {
 
 			inline configcontainer * get_cfg() const { return cfg; }
 
-			void write_item(const rss_item& item, const std::string& filename);
+			void write_item(std::tr1::shared_ptr<rss_item> item, const std::string& filename);
 
 			void mark_deleted(const std::string& guid, bool b);
 
@@ -72,10 +73,22 @@ namespace newsbeuter {
 			void compute_unread_numbers(unsigned int&, unsigned int& );
 			void execute_commands(char ** argv, unsigned int i);
 
+			std::string prepare_message(unsigned int pos, unsigned int max);
+			void save_feed(std::tr1::shared_ptr<rss_feed> feed, unsigned int pos);
+			void enqueue_items(std::tr1::shared_ptr<rss_feed> feed);
+
+			std::string generate_enqueue_filename(const std::string& url, std::tr1::shared_ptr<rss_feed> feed);
+			std::string get_hostname_from_url(const std::string& url);
+
+			void import_read_information(const std::string& readinfofile);
+			void export_read_information(const std::string& readinfofile);
+
+			void sort_feeds();
+
 			view * v;
 			urlreader * urlcfg;
 			cache * rsscache;
-			std::vector<rss_feed> feeds;
+			std::vector<std::tr1::shared_ptr<rss_feed> > feeds;
 			std::string config_dir;
 			std::string url_file;
 			std::string cache_file;
