@@ -4,10 +4,11 @@
 #include <formaction.h>
 #include <history.h>
 #include <mutex.h>
+#include <regexmanager.h>
 
 namespace newsbeuter {
 
-typedef std::pair<rss_item *, unsigned int> itemptr_pos_pair;
+typedef std::pair<std::tr1::shared_ptr<rss_item>, unsigned int> itemptr_pos_pair;
 
 class itemlist_formaction : public formaction {
 	public:
@@ -16,16 +17,11 @@ class itemlist_formaction : public formaction {
 		virtual void prepare();
 		virtual void init();
 
-		inline void set_feed(rss_feed * fd) { 
-			feed = fd; 
-			update_visible_items = true; 
-			do_update_visible_items();
-			apply_filter = false;
-		}
+		void set_feed(std::tr1::shared_ptr<rss_feed> fd);
 
 		virtual std::string id() const { return "articlelist"; }
 
-		inline rss_feed * get_feed() { return feed; }
+		inline std::tr1::shared_ptr<rss_feed> get_feed() { return feed; }
 		inline void set_pos(unsigned int p) { pos = p; }
 		std::string get_guid();
 		virtual keymap_hint_entry * get_keymap_hint();
@@ -45,17 +41,30 @@ class itemlist_formaction : public formaction {
 
 		virtual void recalculate_form();
 
+		void set_regexmanager(regexmanager * r);
+
 	private:
 		virtual void process_operation(operation op, bool automatic = false, std::vector<std::string> * args = NULL);
 		void set_head(const std::string& s, unsigned int unread, unsigned int total, const std::string &url);
 		int get_pos(unsigned int idx);
 
-		void save_article(const std::string& filename, const rss_item& item);
+		void save_article(const std::string& filename, std::tr1::shared_ptr<rss_item> item);
 
 		void save_filterpos();
 
+		void qna_end_setfilter();
+		void qna_end_editflags();
+		void qna_start_search();
+
+		void handle_cmdline_num(unsigned int idx);
+
+		std::string gen_flags(std::tr1::shared_ptr<rss_item> item);
+		std::string gen_datestr(time_t t, const char * datetimeformat);
+
+		void prepare_set_filterpos();
+
 		unsigned int pos;
-		rss_feed * feed;
+		std::tr1::shared_ptr<rss_feed> feed;
 		bool rebuild_list;
 		bool apply_filter;
 		matcher m;
@@ -65,12 +74,16 @@ class itemlist_formaction : public formaction {
 
 		history filterhistory;
 
-		rss_feed search_dummy_feed;
+		std::tr1::shared_ptr<rss_feed> search_dummy_feed;
 
 		mutex redraw_mtx;
 
 		bool set_filterpos;
 		unsigned int filterpos;
+
+		regexmanager * rxman;
+
+		unsigned int old_width;
 };
 
 }
