@@ -7,6 +7,7 @@
 #include <configcontainer.h>
 #include <matcher.h>
 
+#include <tr1/memory>
 
 extern "C" {
 #include <_mrss.h>
@@ -21,8 +22,8 @@ namespace newsbeuter {
 
 	class rss_item : public matchable {
 		public:
-			rss_item(cache * c) : unread_(true), ch(c), enqueued_(false), feedptr(NULL), deleted_(0) { }
-			~rss_item() { }
+			rss_item(cache * c);
+			~rss_item();
 			
 			std::string title() const;
 			std::string title_raw() const { return title_; }
@@ -54,7 +55,7 @@ namespace newsbeuter {
 			inline bool unread() const { return unread_; }
 			void set_unread(bool u);
 			void set_unread_nowrite(bool u);
-			void set_unread_nowrite_notify(bool u);
+			void set_unread_nowrite_notify(bool u, bool notify);
 			
 			inline void set_cache(cache * c) { ch = c; }
 			inline void set_feedurl(const std::string& f) { feedurl_ = f; }
@@ -78,8 +79,8 @@ namespace newsbeuter {
 			virtual bool has_attribute(const std::string& attribname);
 			virtual std::string get_attribute(const std::string& attribname);
 
-			void set_feedptr(rss_feed * ptr);
-			inline rss_feed * get_feedptr() { return feedptr; }
+			void set_feedptr(std::tr1::shared_ptr<rss_feed> ptr);
+			inline std::tr1::shared_ptr<rss_feed> get_feedptr() { return feedptr; }
 
 			inline bool deleted() const { return deleted_; }
 			inline void set_deleted(bool b) { deleted_ = b; }
@@ -98,15 +99,15 @@ namespace newsbeuter {
 			std::string enclosure_type_;
 			bool enqueued_;
 			std::string flags_;
-			rss_feed * feedptr;
+			std::tr1::shared_ptr<rss_feed> feedptr;
 			bool deleted_;
 	};
 
 	class rss_feed : public matchable {
 		public:
-			rss_feed(cache * c) : ch(c), empty(true), is_rtl_(false) { }
-			rss_feed() : ch(NULL), empty(true), is_rtl_(false) { }
-			~rss_feed() { }
+			rss_feed(cache * c);
+			rss_feed();
+			~rss_feed();
 			std::string title_raw() const { return title_; }
 			std::string title() const;
 			inline void set_title(const std::string& t) { title_ = t; }
@@ -121,9 +122,9 @@ namespace newsbeuter {
 			inline std::string pubDate() const { return "TODO"; }
 			inline void set_pubDate(time_t t) { pubDate_ = t; }
 			
-			inline std::vector<rss_item>& items() { return items_; }
+			inline std::vector<std::tr1::shared_ptr<rss_item> >& items() { return items_; }
 
-			rss_item& get_item_by_guid(const std::string& guid);
+			std::tr1::shared_ptr<rss_item> get_item_by_guid(const std::string& guid);
 			
 			inline const std::string& rssurl() const { return rssurl_; }
 			void set_rssurl(const std::string& u);
@@ -133,11 +134,12 @@ namespace newsbeuter {
 			void set_tags(const std::vector<std::string>& tags);
 			bool matches_tag(const std::string& tag);
 			std::string get_tags();
+			std::string get_firsttag();
 
 			virtual bool has_attribute(const std::string& attribname);
 			virtual std::string get_attribute(const std::string& attribname);
 
-			void update_items(std::vector<rss_feed>& feeds);
+			void update_items(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds);
 
 			inline void set_query(const std::string& s) { query = s; }
 
@@ -159,7 +161,7 @@ namespace newsbeuter {
 			std::string link_;
 			time_t pubDate_;
 			std::string rssurl_;
-			std::vector<rss_item> items_;
+			std::vector<std::tr1::shared_ptr<rss_item> > items_;
 			std::vector<std::string> tags_;
 			std::string query;
 			
@@ -169,31 +171,18 @@ namespace newsbeuter {
 			bool is_rtl_;
 	};
 
-	class rss_ignores;
-
-	class rss_parser {
-		public:
-			rss_parser(const char * uri, cache * c, configcontainer *, rss_ignores * ii);
-			~rss_parser();
-			rss_feed parse();
-			static time_t parse_date(const std::string& datestr);
-			bool check_and_update_lastmodified();
-		private:
-			void replace_newline_characters(std::string& str);
-			std::string my_uri;
-			cache * ch;
-			configcontainer *cfgcont;
-			mrss_t * mrss;
-			rss_ignores * ign;
-	};
-
 	class rss_ignores : public config_action_handler {
 		public:
+			rss_ignores() { }
 			virtual ~rss_ignores();
 			virtual action_handler_status handle_action(const std::string& action, const std::vector<std::string>& params);
-			bool matches(rss_item* item);
+			bool matches(rss_item * item);
+			bool matches_lastmodified(const std::string& url);
+			bool matches_resetunread(const std::string& url);
 		private:
 			std::vector<feedurl_expr_pair> ignores;
+			std::vector<std::string> ignores_lastmodified;
+			std::vector<std::string> resetflag;
 	};
 
 }
