@@ -25,10 +25,9 @@ class utils {
 
 		static std::string get_command_output(const std::string& cmd);
 		static void extract_filter(const std::string& line, std::string& filter, std::string& url);
-		static std::string retrieve_url(const std::string& url, const char * user_agent = 0, const char * auth = 0);
-		static std::string run_filter(const std::string& cmd, const std::string& input);
+		static std::string retrieve_url(const std::string& url, const char * user_agent = 0, const char * auth = 0, int download_timeout = 30);
 		static void run_command(const std::string& cmd, const std::string& param); // used for notifications only
-		static std::string run_program(char * argv[], const std::string& input); // used for bookmarking
+		static std::string run_program(char * argv[], const std::string& input);
 
 		static std::string resolve_tilde(const std::string& );
 		static std::string replace_all(std::string str, const std::string& from, const std::string& to);
@@ -47,6 +46,19 @@ class utils {
 		static std::string get_useragent(configcontainer * cfgcont);
 
 		static size_t strwidth(const std::string& s);
+
+		static inline unsigned int max(unsigned int a, unsigned int b) { return (a > b) ? a : b; }
+
+		static unsigned int to_u(const std::string& str);
+
+		static bool is_valid_color(const std::string& color);
+		static bool is_valid_attribute(const std::string& attrib);
+
+		static std::vector<std::pair<unsigned int, unsigned int> > partition_indexes(unsigned int start, unsigned int end, unsigned int parts);
+
+	private:
+		static void append_escapes(std::string& str, char c);
+
 };
 
 class scope_measure {
