@@ -11,6 +11,7 @@
 #include <mutex.h>
 #include <htmlrenderer.h>
 #include <colormanager.h>
+#include <regexmanager.h>
 
 #include <stflpp.h>
 #include <filebrowser_formaction.h>
@@ -34,8 +35,8 @@ namespace newsbeuter {
 
 			std::string id() const;
 
-			void set_feedlist(std::vector<rss_feed>& feeds);
-			void update_visible_feeds(std::vector<rss_feed>& feeds);
+			void set_feedlist(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds);
+			void update_visible_feeds(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds);
 			void set_keymap(keymap * k);
 			void set_config_container(configcontainer * cfgcontainer);
 			void show_error(const std::string& msg);
@@ -45,16 +46,17 @@ namespace newsbeuter {
 			inline configcontainer * get_cfg() { return cfg; }
 			inline keymap * get_keys() { return keys; }
 			void set_tags(const std::vector<std::string>& t);
+			void push_empty_formaction();
 			void pop_current_formaction();
 			inline unsigned int formaction_stack_size() { return formaction_stack.size(); }
 			char confirm(const std::string& prompt, const std::string& charset);
 
 			void push_itemlist(unsigned int pos);
-			void push_itemlist(rss_feed * feed);
-			void push_itemview(rss_feed * f, const std::string& guid);
+			void push_itemlist(std::tr1::shared_ptr<rss_feed> feed);
+			void push_itemview(std::tr1::shared_ptr<rss_feed> f, const std::string& guid);
 			void push_help();
 			void push_urlview(const std::vector<linkpair>& links);
-			void push_searchresult(rss_feed * feed);
+			void push_searchresult(std::tr1::shared_ptr<rss_feed> feed);
 
 			std::string run_filebrowser(filebrowser_type type, const std::string& default_filename = "", const std::string& dir = "");
 			std::string select_tag(const std::vector<std::string>& tags);
@@ -70,14 +72,16 @@ namespace newsbeuter {
 			bool get_next_unread_feed();
 			bool get_prev_unread_feed();
 
-			void set_colors(const colormanager& colorman);
+			void set_colors(std::map<std::string,std::string>& fg_colors, std::map<std::string,std::string>& bg_colors, std::map<std::string,std::vector<std::string> >& attributes);
 
-			void notify_itemlist_change(rss_feed& feed);
+			void notify_itemlist_change(std::tr1::shared_ptr<rss_feed>& feed);
 
 			std::string ask_user(const std::string& prompt);
 
 			void feedlist_mark_pos_if_visible(unsigned int pos);
 
+			void set_regexmanager(regexmanager * r);
+
 		protected:
 
 			void set_bindings();
@@ -97,7 +101,6 @@ namespace newsbeuter {
 			filebrowser_formaction * filebrowser;
 			urlview_formaction * urlview;
 			select_formaction * selecttag;
-			itemlist_formaction * searchresult;
 
 			std::list<formaction *> formaction_stack;
 
