@@ -50,16 +50,24 @@ static int single_int_callback(void * handler, int argc, char ** argv, char ** /
 	return 0;
 }
 
+static int single_string_callback(void * handler, int argc, char ** argv, char ** /* azColName */) {
+	std::string * value = reinterpret_cast<std::string *>(handler);
+	if (argc>0 && argv[0]) {
+		*value = argv[0];
+	}
+	return 0;
+}
+
 static int rssfeed_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
-	rss_feed * feed = static_cast<rss_feed *>(myfeed);
+	std::tr1::shared_ptr<rss_feed>* feed = static_cast<std::tr1::shared_ptr<rss_feed>* >(myfeed);
 	// normaly, this shouldn't happen, but we keep the assert()s here nevertheless
 	assert(argc == 3);
 	assert(argv[0] != NULL);
 	assert(argv[1] != NULL);
 	assert(argv[2] != NULL);
-	feed->set_title(argv[0]);
-	feed->set_link(argv[1]);
-	feed->set_rtl(strcmp(argv[2],"1")==0);
+	(*feed)->set_title(argv[0]);
+	(*feed)->set_link(argv[1]);
+	(*feed)->set_rtl(strcmp(argv[2],"1")==0);
 	GetLogger().log(LOG_INFO, "rssfeed_callback: title = %s link = %s is_rtl = %s",argv[0],argv[1], argv[2]);
 	return 0;
 }
@@ -74,85 +82,85 @@ static int vectorofstring_callback(void * vp, int argc, char ** argv, char ** /*
 }
 
 static int rssitem_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
-	rss_feed * feed = static_cast<rss_feed *>(myfeed);
+	std::tr1::shared_ptr<rss_feed>* feed = static_cast<std::tr1::shared_ptr<rss_feed>* >(myfeed);
 	assert (argc == 12);
-	rss_item item(NULL);
-	item.set_guid(argv[0]);
-	item.set_title(argv[1]);
+	std::tr1::shared_ptr<rss_item> item(new rss_item(NULL));
+	item->set_guid(argv[0]);
+	item->set_title(argv[1]);
 	GetLogger().log(LOG_DEBUG, "rssitem_callback: title = %s", argv[1]);
-	item.set_author(argv[2]);
-	item.set_link(argv[3]);
+	item->set_author(argv[2]);
+	item->set_link(argv[3]);
 	
 	std::istringstream is(argv[4]);
 	time_t t;
 	is >> t;
-	item.set_pubDate(t);
+	item->set_pubDate(t);
 	
-	item.set_description(argv[5]);
-	item.set_unread((std::string("1") == argv[6]));
+	item->set_description(argv[5]);
+	item->set_unread((std::string("1") == argv[6]));
 
-	item.set_feedurl(argv[7]);
+	item->set_feedurl(argv[7]);
 
-	item.set_enclosure_url(argv[8] ? argv[8] : "");
-	item.set_enclosure_type(argv[9] ? argv[9] : "");
-	item.set_enqueued((std::string("1") == (argv[10] ? argv[10] : "")));
-	item.set_flags(argv[11] ? argv[11] : "");
+	item->set_enclosure_url(argv[8] ? argv[8] : "");
+	item->set_enclosure_type(argv[9] ? argv[9] : "");
+	item->set_enqueued((std::string("1") == (argv[10] ? argv[10] : "")));
+	item->set_flags(argv[11] ? argv[11] : "");
 
-	feed->items().push_back(item);
+	(*feed)->items().push_back(item);
 	return 0;
 }
 
 static int rssitemvector_callback(void * vector, int argc, char ** argv, char ** /* azColName */) {
-	std::vector<rss_item> * items = static_cast<std::vector<rss_item> *>(vector);
+	std::vector<std::tr1::shared_ptr<rss_item> >* items = static_cast<std::vector<std::tr1::shared_ptr<rss_item> > *>(vector);
 
 	assert (argc == 12);
-	rss_item item(NULL);
-	item.set_guid(argv[0]);
-	item.set_title(argv[1]);
-	item.set_author(argv[2]);
-	item.set_link(argv[3]);
+	std::tr1::shared_ptr<rss_item> item(new rss_item(NULL));
+	item->set_guid(argv[0]);
+	item->set_title(argv[1]);
+	item->set_author(argv[2]);
+	item->set_link(argv[3]);
 	
 	std::istringstream is(argv[4]);
 	time_t t;
 	is >> t;
-	item.set_pubDate(t);
+	item->set_pubDate(t);
 	
-	item.set_description(argv[5]);
-	item.set_unread((std::string("1") == argv[6]));
+	item->set_description(argv[5]);
+	item->set_unread((std::string("1") == argv[6]));
 
-	item.set_feedurl(argv[7]);
+	item->set_feedurl(argv[7]);
 
-	item.set_enclosure_url(argv[8] ? argv[8] : "");
-	item.set_enclosure_type(argv[9] ? argv[9] : "");
-	item.set_enqueued((std::string("1") == (argv[10] ? argv[10] : "")));
-	item.set_flags(argv[11] ? argv[11] : "");
+	item->set_enclosure_url(argv[8] ? argv[8] : "");
+	item->set_enclosure_type(argv[9] ? argv[9] : "");
+	item->set_enqueued((std::string("1") == (argv[10] ? argv[10] : "")));
+	item->set_flags(argv[11] ? argv[11] : "");
 
 	items->push_back(item);
 	return 0;
 }
 
 static int search_item_callback(void * myfeed, int argc, char ** argv, char ** /* azColName */) {
-	std::vector<rss_item> * items = static_cast<std::vector<rss_item> *>(myfeed);
+	std::vector<std::tr1::shared_ptr<rss_item> > * items = static_cast<std::vector<std::tr1::shared_ptr<rss_item> > *>(myfeed);
 	assert (argc == 12);
-	rss_item item(NULL);
-	item.set_guid(argv[0]);
-	item.set_title(argv[1]);
-	item.set_author(argv[2]);
-	item.set_link(argv[3]);
+	std::tr1::shared_ptr<rss_item> item(new rss_item(NULL));
+	item->set_guid(argv[0]);
+	item->set_title(argv[1]);
+	item->set_author(argv[2]);
+	item->set_link(argv[3]);
 	
 	std::istringstream is(argv[4]);
 	time_t t;
 	is >> t;
-	item.set_pubDate(t);
+	item->set_pubDate(t);
 	
-	item.set_description(argv[5]);
-	item.set_unread((std::string("1") == argv[6]));
-	item.set_feedurl(argv[7]);
+	item->set_description(argv[5]);
+	item->set_unread((std::string("1") == argv[6]));
+	item->set_feedurl(argv[7]);
 
-	item.set_enclosure_url(argv[8] ? argv[8] : "");
-	item.set_enclosure_type(argv[9] ? argv[9] : "");
-	item.set_enqueued((std::string("1") == argv[10]));
-	item.set_flags(argv[11] ? argv[11] : "");
+	item->set_enclosure_url(argv[8] ? argv[8] : "");
+	item->set_enclosure_type(argv[9] ? argv[9] : "");
+	item->set_enqueued((std::string("1") == argv[10]));
+	item->set_flags(argv[11] ? argv[11] : "");
 
 	items->push_back(item);
 	return 0;
@@ -318,9 +326,9 @@ std::vector<std::string> cache::get_feed_urls() {
 
 
 // this function writes an rss_feed including all rss_items to the database
-void cache::externalize_rssfeed(rss_feed& feed) {
+void cache::externalize_rssfeed(std::tr1::shared_ptr<rss_feed> feed, bool reset_unread) {
 	scope_measure m1("cache::externalize_feed");
-	if (feed.rssurl().substr(0,6) == "query:")
+	if (feed->rssurl().substr(0,6) == "query:")
 		return;
 
 	sqlite3_exec(db, "BEGIN;", NULL, NULL, NULL);
@@ -328,17 +336,19 @@ void cache::externalize_rssfeed(rss_feed& feed) {
 	{
 		scope_mutex lock(mtx);
 
-		std::string countquery = prepare_query("SELECT count(*) FROM rss_feed WHERE rssurl = '%q';", feed.rssurl().c_str());
+		std::string countquery = prepare_query("SELECT count(*) FROM rss_feed WHERE rssurl = '%q';", feed->rssurl().c_str());
 		cb_handler count_cbh;
 		int rc = sqlite3_exec(db,countquery.c_str(),count_callback,&count_cbh,NULL);
 		int count = count_cbh.count();
-		GetLogger().log(LOG_DEBUG, "cache::externalize_rss_feed: rss_feeds with rssurl = '%s': found %d",feed.rssurl().c_str(), count);
+		GetLogger().log(LOG_DEBUG, "cache::externalize_rss_feed: rss_feeds with rssurl = '%s': found %d",feed->rssurl().c_str(), count);
 		if (count > 0) {
-			std::string updatequery = prepare_query("UPDATE rss_feed SET title = '%q', url = '%q', is_rtl = %u WHERE rssurl = '%q';",feed.title_raw().c_str(),feed.link().c_str(), feed.is_rtl() ? 1 : 0, feed.rssurl().c_str());
+			std::string updatequery = prepare_query("UPDATE rss_feed SET title = '%q', url = '%q', is_rtl = %u WHERE rssurl = '%q';",
+				feed->title_raw().c_str(),feed->link().c_str(), feed->is_rtl() ? 1 : 0, feed->rssurl().c_str());
 			rc = sqlite3_exec(db,updatequery.c_str(),NULL,NULL,NULL);
 			GetLogger().log(LOG_DEBUG,"ran SQL statement: %s", updatequery.c_str());
 		} else {
-			std::string insertquery = prepare_query("INSERT INTO rss_feed (rssurl, url, title, is_rtl) VALUES ( '%q', '%q', '%q', %u );", feed.rssurl().c_str(), feed.link().c_str(), feed.title_raw().c_str(), feed.is_rtl() ? 1 : 0);
+			std::string insertquery = prepare_query("INSERT INTO rss_feed (rssurl, url, title, is_rtl) VALUES ( '%q', '%q', '%q', %u );", 
+				feed->rssurl().c_str(), feed->link().c_str(), feed->title_raw().c_str(), feed->is_rtl() ? 1 : 0);
 			rc = sqlite3_exec(db,insertquery.c_str(),NULL,NULL,NULL);
 			GetLogger().log(LOG_DEBUG,"ran SQL statement: %s", insertquery.c_str());
 		}
@@ -346,34 +356,34 @@ void cache::externalize_rssfeed(rss_feed& feed) {
 	
 	unsigned int max_items = cfg->get_configvalue_as_int("max-items");
 
-	GetLogger().log(LOG_INFO, "cache::externalize_feed: max_items = %u feed.items().size() = %u", max_items, feed.items().size());
+	GetLogger().log(LOG_INFO, "cache::externalize_feed: max_items = %u feed.items().size() = %u", max_items, feed->items().size());
 	
-	if (max_items > 0 && feed.items().size() > max_items) {
-		std::vector<rss_item>::iterator it=feed.items().begin();
+	if (max_items > 0 && feed->items().size() > max_items) {
+		std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin();
 		for (unsigned int i=0;i<max_items;++i)
 			++it;	
-		if (it != feed.items().end())
-			feed.items().erase(it, feed.items().end()); // delete entries that are too much
+		if (it != feed->items().end())
+			feed->items().erase(it, feed->items().end()); // delete entries that are too much
 	}
 
 	// the reverse iterator is there for the sorting foo below (think about it)
-	for (std::vector<rss_item>::reverse_iterator it=feed.items().rbegin(); it != feed.items().rend(); ++it) {
-		update_rssitem(*it, feed.rssurl());
+	for (std::vector<std::tr1::shared_ptr<rss_item> >::reverse_iterator it=feed->items().rbegin(); it != feed->items().rend(); ++it) {
+		update_rssitem(*it, feed->rssurl(), reset_unread);
 	}
 	sqlite3_exec(db, "END;", NULL, NULL, NULL);
 }
 
 // this function reads an rss_feed including all of its rss_items.
 // the feed parameter needs to have the rssurl member set.
-void cache::internalize_rssfeed(rss_feed& feed) {
+void cache::internalize_rssfeed(std::tr1::shared_ptr<rss_feed> feed) {
 	scope_measure m1("cache::internalize_rssfeed");
-	if (feed.rssurl().substr(0,6) == "query:")
+	if (feed->rssurl().substr(0,6) == "query:")
 		return;
 
 	scope_mutex lock(mtx);
 
 	/* first, we check whether the feed is there at all */
-	std::string query = prepare_query("SELECT count(*) FROM rss_feed WHERE rssurl = '%q';",feed.rssurl().c_str());
+	std::string query = prepare_query("SELECT count(*) FROM rss_feed WHERE rssurl = '%q';",feed->rssurl().c_str());
 	cb_handler count_cbh;
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
@@ -387,7 +397,7 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 	}
 
 	/* then we first read the feed from the database */
-	query = prepare_query("SELECT title, url, is_rtl FROM rss_feed WHERE rssurl = '%q';",feed.rssurl().c_str());
+	query = prepare_query("SELECT title, url, is_rtl FROM rss_feed WHERE rssurl = '%q';",feed->rssurl().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 	rc = sqlite3_exec(db,query.c_str(),rssfeed_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
@@ -395,12 +405,10 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 		throw dbexception(db);
 	}
 
-	if (feed.items().size() > 0) {
-		feed.items().erase(feed.items().begin(),feed.items().end());
-	}
+	feed->items().clear();
 
 	/* ...and then the associated items */
-	query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE feedurl = '%q' AND deleted = 0 ORDER BY pubDate DESC, id DESC;",feed.rssurl().c_str());
+	query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags FROM rss_item WHERE feedurl = '%q' AND deleted = 0 ORDER BY pubDate DESC, id DESC;",feed->rssurl().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 	rc = sqlite3_exec(db,query.c_str(),rssitem_callback,&feed,NULL);
 	if (rc != SQLITE_OK) {
@@ -408,35 +416,35 @@ void cache::internalize_rssfeed(rss_feed& feed) {
 		throw dbexception(db);
 	}
 
-	for (std::vector<rss_item>::iterator it=feed.items().begin(); it != feed.items().end(); ++it) {
-		it->set_cache(this);
-		it->set_feedptr(&feed);
-		it->set_feedurl(feed.rssurl());
+	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin(); it != feed->items().end(); ++it) {
+		(*it)->set_cache(this);
+		(*it)->set_feedptr(feed);
+		(*it)->set_feedurl(feed->rssurl());
 	}
 	
 	unsigned int max_items = cfg->get_configvalue_as_int("max-items");
 	
-	if (max_items > 0 && feed.items().size() > max_items) {
-		std::vector<rss_item> flagged_items;
-		std::vector<rss_item>::iterator it=feed.items().begin();
+	if (max_items > 0 && feed->items().size() > max_items) {
+		std::vector<std::tr1::shared_ptr<rss_item> > flagged_items;
+		std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin();
 		for (unsigned int i=0;i<max_items;++i)
 			++it;
-		for (unsigned int i=max_items;i<feed.items().size();++i) {
-			if (feed.items()[i].flags().length() == 0) {
-				delete_item(feed.items()[i]);
+		for (unsigned int i=max_items;i<feed->items().size();++i) {
+			if (feed->items()[i]->flags().length() == 0) {
+				delete_item(feed->items()[i]);
 			} else {
-				flagged_items.push_back(feed.items()[i]);
+				flagged_items.push_back(feed->items()[i]);
 			}
 		}	
-		feed.items().erase(it, feed.items().end()); // delete old entries
+		feed->items().erase(it, feed->items().end()); // delete old entries
 		if (flagged_items.size() > 0) {
-			feed.items().insert(feed.items().end(), flagged_items.begin(), flagged_items.end()); // if some flagged articles were saved, append them
+			feed->items().insert(feed->items().end(), flagged_items.begin(), flagged_items.end()); // if some flagged articles were saved, append them
 		}
 	}
-	feed.sort(cfg->get_configvalue("article-sort-order"));
+	feed->sort(cfg->get_configvalue("article-sort-order"));
 }
 
-void cache::get_latest_items(std::vector<rss_item>& items, unsigned int limit) {
+void cache::get_latest_items(std::vector<std::tr1::shared_ptr<rss_item> >& items, unsigned int limit) {
 	scope_mutex lock(mtx);
 	std::string query = prepare_query("SELECT guid,title,author,url,pubDate,content,unread,feedurl,enclosure_url,enclosure_type,enqueued,flags "
 									"FROM rss_item WHERE deleted = 0 ORDER BY pubDate DESC, id DESC LIMIT %d;", limit);
@@ -448,8 +456,8 @@ void cache::get_latest_items(std::vector<rss_item>& items, unsigned int limit) {
 	}
 }
 
-rss_feed cache::get_feed_by_url(const std::string& feedurl) {
-	rss_feed feed(this);
+std::tr1::shared_ptr<rss_feed> cache::get_feed_by_url(const std::string& feedurl) {
+	std::tr1::shared_ptr<rss_feed> feed(new rss_feed(this));
 	std::string query;
 	int rc;
 
@@ -467,9 +475,9 @@ rss_feed cache::get_feed_by_url(const std::string& feedurl) {
 	return feed;
 }
 
-std::vector<rss_item> cache::search_for_items(const std::string& querystr, const std::string& feedurl) {
+std::vector<std::tr1::shared_ptr<rss_item> > cache::search_for_items(const std::string& querystr, const std::string& feedurl) {
 	std::string query;
-	std::vector<rss_item> items;
+	std::vector<std::tr1::shared_ptr<rss_item> > items;
 	int rc;
 
 	scope_mutex lock(mtx);
@@ -490,8 +498,8 @@ std::vector<rss_item> cache::search_for_items(const std::string& querystr, const
 	return items;
 }
 
-void cache::delete_item(const rss_item& item) {
-	std::string query = prepare_query("DELETE FROM rss_item WHERE guid = '%q';",item.guid().c_str());
+void cache::delete_item(const std::tr1::shared_ptr<rss_item>& item) {
+	std::string query = prepare_query("DELETE FROM rss_item WHERE guid = '%q';",item->guid().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s",query.c_str());
 	int rc = sqlite3_exec(db,query.c_str(),NULL,NULL,NULL);
 	if (rc != SQLITE_OK) {
@@ -509,7 +517,7 @@ void cache::do_vacuum() {
 	}
 }
 
-void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
+void cache::cleanup_cache(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds) {
 	mtx->lock(); // we don't use the scope_mutex here... see comments below
 
 	/*
@@ -528,8 +536,8 @@ void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
 		unsigned int i = 0;
 		unsigned int feed_size = feeds.size();
 
-		for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it,++i) {
-			std::string name = prepare_query("'%q'",it->rssurl().c_str());
+		for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it=feeds.begin();it!=feeds.end();++it,++i) {
+			std::string name = prepare_query("'%q'",(*it)->rssurl().c_str());
 			list.append(name);
 			if (i < feed_size-1) {
 				list.append(", ");
@@ -572,9 +580,9 @@ void cache::cleanup_cache(std::vector<rss_feed>& feeds) {
 }
 
 /* this function writes an rss_item to the database, also checking whether this item already exists in the database */
-void cache::update_rssitem(rss_item& item, const std::string& feedurl) {
+void cache::update_rssitem(std::tr1::shared_ptr<rss_item>& item, const std::string& feedurl, bool reset_unread) {
 	scope_mutex lock(mtx);
-	std::string query = prepare_query("SELECT count(*) FROM rss_item WHERE guid = '%q';",item.guid().c_str());
+	std::string query = prepare_query("SELECT count(*) FROM rss_item WHERE guid = '%q';",item->guid().c_str());
 	cb_handler count_cbh;
 	GetLogger().log(LOG_DEBUG,"running query: %s", query.c_str());
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
@@ -583,11 +591,28 @@ void cache::update_rssitem(rss_item& item, const std::string& feedurl) {
 		throw dbexception(db);
 	}
 	if (count_cbh.count() > 0) {
+		if (reset_unread) {
+			std::string content;
+			query = prepare_query("SELECT content FROM rss_item WHERE guid = '%q';", item->guid().c_str());
+			rc = sqlite3_exec(db, query.c_str(), single_string_callback, &content, NULL);
+			if (rc != SQLITE_OK) {
+				GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+				throw dbexception(db);
+			}
+			if (content != item->description_raw()) {
+				query = prepare_query("UPDATE rss_item SET unread = 1 WHERE guid = '%q';", item->guid().c_str());
+				rc = sqlite3_exec(db, query.c_str(), NULL, NULL, NULL);
+				if (rc != SQLITE_OK) {
+					GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+					throw dbexception(db);
+				}
+			}
+		}
 		std::string update = prepare_query("UPDATE rss_item SET title = '%q', author = '%q', url = '%q', feedurl = '%q', content = '%q', enclosure_url = '%q', enclosure_type = '%q' WHERE guid = '%q'",
-			item.title_raw().c_str(), item.author_raw().c_str(), item.link().c_str(), 
-			feedurl.c_str(), item.description_raw().c_str(), 
-			item.enclosure_url().c_str(), item.enclosure_type().c_str(),
-			item.guid().c_str());
+			item->title_raw().c_str(), item->author_raw().c_str(), item->link().c_str(), 
+			feedurl.c_str(), item->description_raw().c_str(), 
+			item->enclosure_url().c_str(), item->enclosure_type().c_str(),
+			item->guid().c_str());
 		GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
 		rc = sqlite3_exec(db,update.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
@@ -597,9 +622,9 @@ void cache::update_rssitem(rss_item& item, const std::string& feedurl) {
 	} else {
 		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued) "
 								"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d)",
-								item.guid().c_str(), item.title_raw().c_str(), item.author_raw().c_str(), 
-								item.link().c_str(), feedurl.c_str(), item.pubDate_timestamp(), item.description_raw().c_str(),
-								item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0);
+								item->guid().c_str(), item->title_raw().c_str(), item->author_raw().c_str(), 
+								item->link().c_str(), feedurl.c_str(), item->pubDate_timestamp(), item->description_raw().c_str(),
+								item->enclosure_url().c_str(), item->enclosure_type().c_str(), item->enqueued() ? 1 : 0);
 		GetLogger().log(LOG_DEBUG,"running query: %s", insert.c_str());
 		rc = sqlite3_exec(db,insert.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
@@ -609,12 +634,12 @@ void cache::update_rssitem(rss_item& item, const std::string& feedurl) {
 	}
 }
 
-void cache::catchup_all(rss_feed& feed) {
+void cache::catchup_all(std::tr1::shared_ptr<rss_feed> feed) {
 	scope_mutex lock(mtx);
 	std::string query = "UPDATE rss_item SET unread = '0' WHERE unread != '0' AND guid IN (";
 
-	for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
-		query.append(prepare_query("'%q',", it->guid().c_str()));
+	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin();it!=feed->items().end();++it) {
+		query.append(prepare_query("'%q',", (*it)->guid().c_str()));
 	}
 	query.append("'');");
 
@@ -644,11 +669,10 @@ void cache::catchup_all(const std::string& feedurl) {
 	}
 }
 
-/* this function updates the unread and enqueued flags */
-void cache::update_rssitem_unread_and_enqueued(rss_item& item, const std::string& feedurl) {
+void cache::update_rssitem_unread_and_enqueued(rss_item* item, const std::string& feedurl) {
 	scope_mutex lock(mtx);
 
-	std::string query = prepare_query("SELECT count(*) FROM rss_item WHERE guid = '%q';",item.guid().c_str());
+	std::string query = prepare_query("SELECT count(*) FROM rss_item WHERE guid = '%q';",item->guid().c_str());
 	cb_handler count_cbh;
 	GetLogger().log(LOG_DEBUG,"running query: %s", query.c_str());
 	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
@@ -659,7 +683,7 @@ void cache::update_rssitem_unread_and_enqueued(rss_item& item, const std::string
 
 	if (count_cbh.count() > 0) {
 		std::string update = prepare_query("UPDATE rss_item SET unread = '%d', enqueued = '%d' WHERE guid = '%q'",
-			item.unread()?1:0, item.enqueued()?1:0, item.guid().c_str());
+			item->unread()?1:0, item->enqueued()?1:0, item->guid().c_str());
 		GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
 		rc = sqlite3_exec(db,update.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
@@ -668,10 +692,47 @@ void cache::update_rssitem_unread_and_enqueued(rss_item& item, const std::string
 		}
 	} else {
 		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued,flags) "
-										"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d)",
-										item.guid().c_str(), item.title_raw().c_str(), item.author_raw().c_str(), 
-										item.link().c_str(), feedurl.c_str(), item.pubDate_timestamp(), item.description_raw().c_str(),
-										item.enclosure_url().c_str(), item.enclosure_type().c_str(), item.enqueued() ? 1 : 0);
+										"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d, '%q')",
+										item->guid().c_str(), item->title_raw().c_str(), item->author_raw().c_str(), 
+										item->link().c_str(), feedurl.c_str(), item->pubDate_timestamp(), item->description_raw().c_str(),
+										item->enclosure_url().c_str(), item->enclosure_type().c_str(), item->enqueued() ? 1 : 0, item->flags().c_str());
+		GetLogger().log(LOG_DEBUG,"running query: %s", insert.c_str());
+		rc = sqlite3_exec(db,insert.c_str(),NULL,NULL,NULL);
+		if (rc != SQLITE_OK) {
+			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", insert.c_str(), rc);
+			throw dbexception(db);
+		}
+	}
+}
+
+/* this function updates the unread and enqueued flags */
+void cache::update_rssitem_unread_and_enqueued(std::tr1::shared_ptr<rss_item>& item, const std::string& feedurl) {
+	scope_mutex lock(mtx);
+
+	std::string query = prepare_query("SELECT count(*) FROM rss_item WHERE guid = '%q';",item->guid().c_str());
+	cb_handler count_cbh;
+	GetLogger().log(LOG_DEBUG,"running query: %s", query.c_str());
+	int rc = sqlite3_exec(db,query.c_str(),count_callback,&count_cbh,NULL);
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", query.c_str(), rc);
+		throw dbexception(db);
+	}
+
+	if (count_cbh.count() > 0) {
+		std::string update = prepare_query("UPDATE rss_item SET unread = '%d', enqueued = '%d' WHERE guid = '%q'",
+			item->unread()?1:0, item->enqueued()?1:0, item->guid().c_str());
+		GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
+		rc = sqlite3_exec(db,update.c_str(),NULL,NULL,NULL);
+		if (rc != SQLITE_OK) {
+			GetLogger().log(LOG_CRITICAL,"query \"%s\" failed: error = %d", update.c_str(), rc);
+			throw dbexception(db);
+		}
+	} else {
+		std::string insert = prepare_query("INSERT INTO rss_item (guid,title,author,url,feedurl,pubDate,content,unread,enclosure_url,enclosure_type,enqueued,flags) "
+										"VALUES ('%q','%q','%q','%q','%q','%u','%q',1,'%q','%q',%d, '%q')",
+										item->guid().c_str(), item->title_raw().c_str(), item->author_raw().c_str(), 
+										item->link().c_str(), feedurl.c_str(), item->pubDate_timestamp(), item->description_raw().c_str(),
+										item->enclosure_url().c_str(), item->enclosure_type().c_str(), item->enqueued() ? 1 : 0, item->flags().c_str());
 		GetLogger().log(LOG_DEBUG,"running query: %s", insert.c_str());
 		rc = sqlite3_exec(db,insert.c_str(),NULL,NULL,NULL);
 		if (rc != SQLITE_OK) {
@@ -695,10 +756,22 @@ std::string cache::prepare_query(const char * format, ...) {
 	return result;
 }
 
-void cache::update_rssitem_flags(rss_item& item) {
+void cache::update_rssitem_flags(rss_item* item) {
 	scope_mutex lock(mtx);
 
-	std::string update = prepare_query("UPDATE rss_item SET flags = '%q' WHERE guid = '%q';", item.flags().c_str(), item.guid().c_str());
+	std::string update = prepare_query("UPDATE rss_item SET flags = '%q' WHERE guid = '%q';", item->flags().c_str(), item->guid().c_str());
+	GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
+	int rc = sqlite3_exec(db,update.c_str(), NULL, NULL, NULL);
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL, "query \"%s\" failed: error = %d", update.c_str(), rc);
+		throw dbexception(db);
+	}
+}
+
+void cache::update_rssitem_flags(std::tr1::shared_ptr<rss_item>& item) {
+	scope_mutex lock(mtx);
+
+	std::string update = prepare_query("UPDATE rss_item SET flags = '%q' WHERE guid = '%q';", item->flags().c_str(), item->guid().c_str());
 	GetLogger().log(LOG_DEBUG,"running query: %s", update.c_str());
 	int rc = sqlite3_exec(db,update.c_str(), NULL, NULL, NULL);
 	if (rc != SQLITE_OK) {
@@ -734,3 +807,36 @@ unsigned int cache::get_unread_count() {
 	GetLogger().log(LOG_DEBUG, "cache::get_unread_count: rc = %d count = %u", rc, count);
 	return count;
 }
+
+void cache::mark_items_read_by_guid(const std::vector<std::string> guids) {
+	scope_measure m1("cache::mark_items_read_by_guid");
+	std::string guidset("(");
+	for (std::vector<std::string>::const_iterator it=guids.begin();it!=guids.end();it++) {
+		guidset.append(prepare_query("'%q', ", it->c_str()));
+	}
+	guidset.append("'')");
+
+	std::string updatequery = utils::strprintf("UPDATE rss_item SET unread = 0 WHERE unread = 1 AND guid IN %s;", guidset.c_str());
+	int rc = sqlite3_exec(db, updatequery.c_str(), NULL, NULL, NULL);
+
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL, "query \"%s\" failed: error = %d", updatequery.c_str(), rc);
+		throw dbexception(db);
+	} else {
+		GetLogger().log(LOG_DEBUG, "cache::mark_items_read_by_guid: executed query successfully: %s", updatequery.c_str());
+	}
+}
+
+std::vector<std::string> cache::get_read_item_guids() {
+	std::vector<std::string> guids;
+	std::string query = "SELECT guid FROM rss_item WHERE unread = 0;";
+
+	int rc = sqlite3_exec(db, query.c_str(), vectorofstring_callback, &guids, NULL);
+	if (rc != SQLITE_OK) {
+		GetLogger().log(LOG_CRITICAL, "query \"%s\" failed: error = %d", query.c_str(), rc);
+		throw dbexception(db);
+	} else {
+		GetLogger().log(LOG_DEBUG, "cache::get_read_item_guids: executed query successfully: %s", query.c_str());
+	}
+	return guids;
+}
