@@ -1,10 +1,10 @@
+#include <config.h>
 #include <configcontainer.h>
 #include <configparser.h>
 #include <logger.h>
 #include <sstream>
 #include <iostream>
 #include <utils.h>
-#include <config.h>
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -53,6 +53,15 @@ configcontainer::configcontainer()
 	config_data["always-display-description"] = configdata("false", configdata::BOOL);
 	config_data["reload-only-visible-feeds"] = configdata("false", configdata::BOOL);
 	config_data["article-sort-order"] = configdata("date", configdata::STR);
+	config_data["show-read-articles"] = configdata("yes", configdata::BOOL);
+	config_data["goto-next-feed"] = configdata("yes", configdata::BOOL);
+	config_data["display-article-progress"] = configdata("yes", configdata::BOOL);
+	config_data["show-keymap-hint"] = configdata("yes", configdata::BOOL);
+	config_data["download-timeout"] = configdata("30", configdata::INT);
+	config_data["download-retries"] = configdata("1", configdata::INT);
+	config_data["feed-sort-order"] = configdata("none", configdata::STR);
+	config_data["reload-threads"] = configdata("1", configdata::INT);
+
 
 	/* undocumented: */
 	config_data["feedlist-title-format"] = configdata(_("%N %V - Your feeds (%u unread, %t total)%?T? - tag `%T'&?"), configdata::STR);
@@ -91,32 +100,25 @@ action_handler_status configcontainer::handle_action(const std::string& action,
 
 	GetLogger().log(LOG_DEBUG, "configcontainer::handle_action: action = %s, type = %u", action.c_str(), cfgdata.type);
 
+	if (params.size() < 1) {
+		return AHS_TOO_FEW_PARAMS;
+	}
+
 	switch (cfgdata.type) {
 		case configdata::BOOL:
-			if (params.size() < 1) {
-				return AHS_TOO_FEW_PARAMS;
-			}
-			if (!is_bool(params[0])) {
+			if (!is_bool(params[0]))
 				return AHS_INVALID_PARAMS;
-			}
 			cfgdata.value = params[0];
 			return AHS_OK; 
 
 		case configdata::INT:
-			if (params.size() < 1) {
-				return AHS_TOO_FEW_PARAMS;
-			}
-			if (!is_int(params[0])) {
+			if (!is_int(params[0]))
 				return AHS_INVALID_PARAMS;
-			}
 			cfgdata.value = params[0];
 			return AHS_OK;	
 
 		case configdata::STR:
 		case configdata::PATH:
-			if (params.size() < 1) {
-				return AHS_TOO_FEW_PARAMS;
-			}
 			cfgdata.value = params[0];
 			return AHS_OK;	
 
