@@ -11,11 +11,16 @@
 #include <stflpp.h>
 #include <exception.h>
 #include <formatstring.h>
+#include <regexmanager.h>
+#include <rss_parser.h>
+
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
 #include <fstream>
 #include <cerrno>
+#include <algorithm>
+#include <functional>
 
 #include <sys/time.h>
 #include <ctime>
@@ -23,6 +28,7 @@
 #include <signal.h>
 #include <sys/utsname.h>
 #include <langinfo.h>
+#include <libgen.h>
 
 #include <_nxml.h>
 
@@ -32,7 +38,7 @@
 
 #include <ncursesw/ncurses.h>
 
-using namespace newsbeuter;
+namespace newsbeuter {
 
 #define LOCK_SUFFIX ".lock"
 
@@ -42,9 +48,6 @@ void ctrl_c_action(int sig) {
 	GetLogger().log(LOG_DEBUG,"caugh signal %d",sig);
 	stfl::reset();
 	::unlink(lock_file.c_str());
-	if (SIGSEGV == sig) {
-		fprintf(stderr,"%s\n", _("Segmentation fault."));
-	}
 	::exit(EXIT_FAILURE);
 }
 
@@ -84,6 +87,11 @@ controller::~controller() {
 	delete reload_mutex;
 	delete cfg;
 	delete urlcfg;
+
+	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it=feeds.begin();it!=feeds.end();it++) {
+		(*it)->items().clear();
+	}
+	feeds.clear();
 }
 
 void controller::set_view(view * vv) {
@@ -100,9 +108,6 @@ void controller::run(int argc, char * argv[]) {
 	queue_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + queue_file;
 
 	::signal(SIGINT, ctrl_c_action);
-#ifndef DEBUG
-	// ::signal(SIGSEGV, ctrl_c_action);
-#endif
 	::signal(SIGPIPE, ignore_signal);
 	::signal(SIGHUP, ctrl_c_action);
 	::signal(SIGCHLD, omg_a_child_died);
@@ -110,12 +115,14 @@ void controller::run(int argc, char * argv[]) {
 	bool do_import = false, do_export = false, cachefile_given_on_cmdline = false, do_vacuum = false;
 	bool offline_mode = false, real_offline_mode = false;
 	std::string importfile;
+	bool do_read_import = false, do_read_export = false;
+	std::string readinfofile;
 
 	bool silent = false;
 	bool execute_cmds = false;
 
 	do {
-		if((c = ::getopt(argc,argv,"i:erhu:c:C:d:l:vVox"))<0)
+		if((c = ::getopt(argc,argv,"i:erhu:c:C:d:l:vVoxI:E:"))<0)
 			continue;
 		switch (c) {
 			case ':': /* fall-through */
@@ -175,6 +182,18 @@ void controller::run(int argc, char * argv[]) {
 						GetLogger().set_loglevel(level);
 				}
 				break;
+			case 'I':
+				if (do_read_export)
+					usage(argv[0]);
+				do_read_import = true;
+				readinfofile = optarg;
+				break;
+			case 'E':
+				if (do_read_import)
+					usage(argv[0]);
+				do_read_export = true;
+				readinfofile = optarg;
+				break;
 			default:
 				std::cout << utils::strprintf(_("%s: unknown option - %c"), argv[0], static_cast<char>(c)) << std::endl;
 				usage(argv[0]);
@@ -190,6 +209,7 @@ void controller::run(int argc, char * argv[]) {
 		return;
 	}
 
+
 	GetLogger().log(LOG_INFO, "nl_langinfo(CODESET): %s", nl_langinfo(CODESET));
 
 	if (!do_export) {
@@ -225,9 +245,15 @@ void controller::run(int argc, char * argv[]) {
 	cfgparser.register_handler("macro", &keys);
 
 	cfgparser.register_handler("ignore-article",&ign);
+	cfgparser.register_handler("always-download",&ign);
+	cfgparser.register_handler("reset-unread-on-update",&ign);
 
 	cfgparser.register_handler("define-filter",&filters);
 
+	regexmanager rxman;
+
+	cfgparser.register_handler("highlight", &rxman);
+
 	try {
 		cfgparser.parse("/etc/" PROGRAM_NAME "/config");
 		cfgparser.parse(config_file);
@@ -238,9 +264,10 @@ void controller::run(int argc, char * argv[]) {
 		return;	
 	}
 
+	v->set_regexmanager(&rxman);
 
 	if (colorman->colors_loaded()) {
-		v->set_colors(*colorman);
+		v->set_colors(colorman->get_fgcolors(), colorman->get_bgcolors(), colorman->get_attributes());
 	}
 	delete colorman;
 
@@ -353,9 +380,9 @@ void controller::run(int argc, char * argv[]) {
 	}
 
 	for (std::vector<std::string>::const_iterator it=urlcfg->get_urls().begin(); it != urlcfg->get_urls().end(); ++it) {
-		rss_feed feed(rsscache);
-		feed.set_rssurl(*it);
-		feed.set_tags(urlcfg->get_tags(*it));
+		std::tr1::shared_ptr<rss_feed> feed(new rss_feed(rsscache));
+		feed->set_rssurl(*it);
+		feed->set_tags(urlcfg->get_tags(*it));
 		try {
 			rsscache->internalize_rssfeed(feed);
 		} catch(const dbexception& e) {
@@ -366,6 +393,8 @@ void controller::run(int argc, char * argv[]) {
 		feeds.push_back(feed);
 	}
 
+	sort_feeds();
+
 	std::vector<std::string> tags = urlcfg->get_alltags();
 
 	if (!do_export && !silent)
@@ -377,6 +406,24 @@ void controller::run(int argc, char * argv[]) {
 		return;
 	}
 
+	if (do_read_import) {
+		GetLogger().log(LOG_INFO,"Importing read information file from %s",readinfofile.c_str());
+		std::cout << _("Importing list of read articles...");
+		std::cout.flush();
+		import_read_information(readinfofile);
+		std::cout << _("done.") << std::endl;
+		return;
+	}
+
+	if (do_read_export) {
+		GetLogger().log(LOG_INFO,"Exporting read information file to %s",readinfofile.c_str());
+		std::cout << _("Exporting list of read articles...");
+		std::cout.flush();
+		export_read_information(readinfofile);
+		std::cout << _("done.") << std::endl;
+		return;
+	}
+
 	if (execute_cmds) {
 		execute_commands(argv, optind);
 		utils::remove_fs_lock(lock_file);
@@ -424,10 +471,10 @@ void controller::catchup_all() {
 		v->show_error(utils::strprintf(_("Error: couldn't mark all feeds read: %s"), e.what()));
 		return;
 	}
-	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
-		if (it->items().size() > 0) {
-			for (std::vector<rss_item>::iterator jt=it->items().begin();jt!=it->items().end();++jt) {
-				jt->set_unread_nowrite(false);
+	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it=feeds.begin();it!=feeds.end();++it) {
+		if ((*it)->items().size() > 0) {
+			for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator jt=(*it)->items().begin();jt!=(*it)->items().end();++jt) {
+				(*jt)->set_unread_nowrite(false);
 			}
 		}
 	}
@@ -435,15 +482,21 @@ void controller::catchup_all() {
 
 void controller::mark_all_read(unsigned int pos) {
 	if (pos < feeds.size()) {
-		rss_feed& feed = feeds[pos];
-		if (feed.rssurl().substr(0,6) == "query:") {
+		scope_measure m("controller::mark_all_read");
+		std::tr1::shared_ptr<rss_feed> feed = feeds[pos];
+		if (feed->rssurl().substr(0,6) == "query:") {
 			rsscache->catchup_all(feed);
 		} else {
-			rsscache->catchup_all(feed.rssurl());
+			rsscache->catchup_all(feed->rssurl());
 		}
-		if (feed.items().size() > 0) {
-			for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
-				it->set_unread_nowrite_notify(false);
+		m.stopover("after rsscache->catchup_all, before iteration over items");
+		std::vector<std::tr1::shared_ptr<rss_item> >& items = feed->items();
+		std::vector<std::tr1::shared_ptr<rss_item> >::iterator begin = items.begin(), end = items.end();
+		if (items.size() > 0) {
+			bool notify = items[0]->feedurl() != feed->rssurl();
+			GetLogger().log(LOG_DEBUG, "controller::mark_all_read: notify = %s", notify ? "yes" : "no");
+			for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=begin;it!=end;++it) {
+				(*it)->set_unread_nowrite_notify(false, notify);
 			}
 		}
 	}
@@ -452,70 +505,36 @@ void controller::mark_all_read(unsigned int pos) {
 void controller::reload(unsigned int pos, unsigned int max, bool unattended) {
 	GetLogger().log(LOG_DEBUG, "controller::reload: pos = %u max = %u", pos, max);
 	if (pos < feeds.size()) {
-		rss_feed feed = feeds[pos];
-		std::string msg;
-		if (max > 0) {
-			msg = utils::strprintf("(%u/%u) ", pos+1, max);
-		}
-		GetLogger().log(LOG_DEBUG, "controller::reload: before setting status");
+		std::tr1::shared_ptr<rss_feed> feed = feeds[pos];
 		if (!unattended)
-			v->set_status(utils::strprintf(_("%sLoading %s..."), msg.c_str(), feed.rssurl().c_str()));
-		GetLogger().log(LOG_DEBUG, "controller::reload: after setting status");
-				
-		rss_parser parser(feed.rssurl().c_str(), rsscache, cfg, &ign);
+			v->set_status(utils::strprintf(_("%sLoading %s..."), prepare_message(pos+1, max).c_str(), feed->rssurl().c_str()));
+
+		rss_parser parser(feed->rssurl().c_str(), rsscache, cfg, &ign);
 		GetLogger().log(LOG_DEBUG, "controller::reload: created parser");
 		try {
 			if (parser.check_and_update_lastmodified()) {
 				feed = parser.parse();
-				GetLogger().log(LOG_DEBUG, "controller::reload: after parser.parse");
-				if (!feed.is_empty()) {
-					GetLogger().log(LOG_DEBUG, "controller::reload: feed is nonempty, saving");
-					rsscache->externalize_rssfeed(feed);
-					GetLogger().log(LOG_DEBUG, "controller::reload: after externalize_rssfeed");
-
-					rsscache->internalize_rssfeed(feed);
-					GetLogger().log(LOG_DEBUG, "controller::reload: after internalize_rssfeed");
-					feed.set_tags(urlcfg->get_tags(feed.rssurl()));
-					feeds[pos] = feed;
-					v->notify_itemlist_change(feed);
-				} else {
-					GetLogger().log(LOG_DEBUG, "controller::reload: feed is empty, not saving");
-				}
-
-				for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
-					if (cfg->get_configvalue_as_bool("podcast-auto-enqueue") && !it->enqueued() && it->enclosure_url().length() > 0) {
-						GetLogger().log(LOG_DEBUG, "controller::reload: enclosure_url = `%s' enclosure_type = `%s'", it->enclosure_url().c_str(), it->enclosure_type().c_str());
-						if (is_valid_podcast_type(it->enclosure_type())) {
-							GetLogger().log(LOG_INFO, "controller::reload: enqueuing `%s'", it->enclosure_url().c_str());
-							enqueue_url(it->enclosure_url());
-							it->set_enqueued(true);
-							rsscache->update_rssitem_unread_and_enqueued(*it, feed.rssurl());
-						}
-					}
-				}
-
-				
-				if (!unattended) {
+				save_feed(feed, pos);
+				enqueue_items(feed);
+				if (!unattended)
 					v->set_feedlist(feeds);
-					GetLogger().log(LOG_DEBUG, "controller::reload: after set_feedlist");
-				}
 			}
 			v->set_status("");
 		} catch (const dbexception& e) {
-			v->set_status(utils::strprintf(_("Error while retrieving %s: %s"), feed.rssurl().c_str(), e.what()));
+			v->set_status(utils::strprintf(_("Error while retrieving %s: %s"), feed->rssurl().c_str(), e.what()));
 		} catch (const std::string& errmsg) {
-			v->set_status(utils::strprintf(_("Error while retrieving %s: %s"), feed.rssurl().c_str(), errmsg.c_str()));
+			v->set_status(utils::strprintf(_("Error while retrieving %s: %s"), feed->rssurl().c_str(), errmsg.c_str()));
 		}
 	} else {
 		v->show_error(_("Error: invalid feed!"));
 	}
 }
 
-rss_feed * controller::get_feed(unsigned int pos) {
+std::tr1::shared_ptr<rss_feed> controller::get_feed(unsigned int pos) {
 	if (pos >= feeds.size()) {
 		throw std::out_of_range(_("invalid feed index (bug)"));
 	}
-	return &(feeds[pos]);
+	return feeds[pos];
 }
 
 void controller::reload_indexes(const std::vector<int>& indexes, bool unattended) {
@@ -541,16 +560,40 @@ void controller::reload_indexes(const std::vector<int>& indexes, bool unattended
 		v->set_status("");
 }
 
+void controller::reload_range(unsigned int start, unsigned int end, unsigned int size, bool unattended) {
+	for (unsigned int i=start;i<=end;i++) {
+		GetLogger().log(LOG_DEBUG, "controller::reload_range: reloading feed #%u", i);
+		this->reload(i, size, unattended);
+	}
+}
+
 void controller::reload_all(bool unattended) {
-	GetLogger().log(LOG_DEBUG,"controller::reload_all: starting with reload all...");
 	unsigned int unread_feeds, unread_articles;
 	compute_unread_numbers(unread_feeds, unread_articles);
+	unsigned int num_threads = cfg->get_configvalue_as_int("reload-threads");
 	time_t t1, t2, dt;
+
 	t1 = time(NULL);
-	for (unsigned int i=0;i<feeds.size();++i) {
-		GetLogger().log(LOG_DEBUG, "controller::reload_all: reloading feed #%u", i);
-		this->reload(i,feeds.size(), unattended);
+
+	GetLogger().log(LOG_DEBUG,"controller::reload_all: starting with reload all...");
+	if (num_threads <= 1) {
+		this->reload_range(0, feeds.size()-1, feeds.size(), unattended);
+	} else {
+		std::vector<std::pair<unsigned int, unsigned int> > partitions = utils::partition_indexes(0, feeds.size()-1, num_threads);
+		std::vector<pthread_t> threads;
+		GetLogger().log(LOG_DEBUG, "controller::reload_all: starting reload threads...");
+		for (unsigned int i=0;i<num_threads-1;i++) {
+			reloadrangethread* t = new reloadrangethread(this, partitions[i].first, partitions[i].second, feeds.size(), unattended);
+			threads.push_back(t->start());
+		}
+		GetLogger().log(LOG_DEBUG, "controller::reload_all: starting my own reload...");
+		this->reload_range(partitions[num_threads-1].first, partitions[num_threads-1].second, feeds.size(), unattended);
+		GetLogger().log(LOG_DEBUG, "controller::reload_all: joining other threads...");
+		for (std::vector<pthread_t>::iterator it=threads.begin();it!=threads.end();it++) {
+			::pthread_join(*it, NULL);
+		}
 	}
+
 	t2 = time(NULL);
 	dt = t2 - t1;
 	GetLogger().log(LOG_INFO, "controller::reload_all: reload took %d seconds", dt);
@@ -588,8 +631,8 @@ void controller::notify(const std::string& msg) {
 void controller::compute_unread_numbers(unsigned int& unread_feeds, unsigned int& unread_articles) {
 	unread_feeds = 0;
 	unread_articles = 0;
-	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
-		unsigned int items = it->unread_item_count();
+	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it=feeds.begin();it!=feeds.end();++it) {
+		unsigned int items = (*it)->unread_item_count();
 		if (items > 0) {
 			++unread_feeds;
 			unread_articles += items;
@@ -630,19 +673,48 @@ void controller::version_information() {
 	::exit(EXIT_SUCCESS);
 }
 
+static unsigned int gentabs(const std::string& str) {
+	int tabcount = 2 - ((3 + utils::strwidth(str)) / 8);
+	if (tabcount <= 0) {
+		tabcount = 1;
+	}
+	return tabcount;
+}
+
 void controller::usage(char * argv0) {
-	std::cout << utils::strprintf(_("%s %s\nusage: %s [-i <file>|-e] [-u <urlfile>] [-c <cachefile>] [-x <command> ...] [-h]\n"
-				"-e              export OPML feed to stdout\n"
-				"-r              refresh feeds on start\n"
-				"-i <file>       import OPML file\n"
-				"-u <urlfile>    read RSS feed URLs from <urlfile>\n"
-				"-c <cachefile>  use <cachefile> as cache file\n"
-				"-C <configfile> read configuration from <configfile>\n"
-				"-v              clean up cache thoroughly\n"
-				"-x <command>... execute list of commands\n"
-				"-o              activate offline mode (only applies to bloglines synchronization mode)\n"
-				"-V              get version information\n"
-				"-h              this help\n"), PROGRAM_NAME, PROGRAM_VERSION, argv0);
+	std::cout << utils::strprintf(_("%s %s\nusage: %s [-i <file>|-e] [-u <urlfile>] [-c <cachefile>] [-x <command> ...] [-h]\n"), 
+					PROGRAM_NAME, PROGRAM_VERSION, argv0);
+	struct {
+		char arg;
+		const char * params;
+		const char * desc;
+	} args[] = {
+		{ 'e', "", _("export OPML feed to stdout") },
+		{ 'r', "", _("refresh feeds on start") },
+		{ 'i', _("<file>"), _("import OPML file") },
+		{ 'u', _("<urlfile>"), _("read RSS feed URLs from <urlfile>") },
+		{ 'c', _("<cachefile>"), _("use <cachefile> as cache file") },
+		{ 'C', _("<configfile>"), _("read configuration from <configfile>") },
+		{ 'v', "", _("clean up cache thoroughly") },
+		{ 'x', _("<command>..."), _("execute list of commands") },
+		{ 'o', "", _("activate offline mode (only applies to bloglines synchronization mode)") },
+		{ 'V', "", _("get version information") },
+		{ 'l', _("<loglevel>"), _("write a log with a certain loglevel (valid values: 1 to 6)") },
+		{ 'd', _("<logfile>"), _("use <logfile> as output log file") },
+		{ 'E', _("<file>"), _("export list of read articles to <file>") },
+		{ 'I', _("<file>"), _("import list of read articles from <file>") },
+		{ 'h', "", _("this help") },
+		{ '\0', NULL, NULL }
+	};
+
+	for (unsigned int i=0;args[i].arg != '\0';i++) {
+		unsigned int tabs = gentabs(args[i].params);
+		std::cout << "\t-" << args[i].arg << " " << args[i].params;
+		for (unsigned int j=0;j<tabs;j++) { 
+			std::cout << "\t";
+		}
+		std::cout << args[i].desc << std::endl;
+	}
 	::exit(EXIT_FAILURE);
 }
 
@@ -683,11 +755,11 @@ void controller::export_opml() {
 	std::cout << "<opml version=\"1.0\">" << std::endl;
 	std::cout << "\t<head>" << std::endl << "\t\t<title>" PROGRAM_NAME " - Exported Feeds</title>" << std::endl << "\t</head>" << std::endl;
 	std::cout << "\t<body>" << std::endl;
-	for (std::vector<rss_feed>::iterator it=feeds.begin(); it != feeds.end(); ++it) {
-		if (it->rssurl().substr(0,6) != "query:" && it->rssurl().substr(0,7) != "filter:") {
-			std::string rssurl = utils::replace_all(it->rssurl(),"&","&amp;");
-			std::string link = utils::replace_all(it->link(),"&", "&amp;");
-			std::string title = utils::replace_all(it->title(),"&", "&amp;");
+	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it=feeds.begin(); it != feeds.end(); ++it) {
+		if ((*it)->rssurl().substr(0,6) != "query:" && (*it)->rssurl().substr(0,7) != "filter:") {
+			std::string rssurl = utils::replace_all((*it)->rssurl(),"&","&amp;");
+			std::string link = utils::replace_all((*it)->link(),"&", "&amp;");
+			std::string title = utils::replace_all((*it)->title(),"&", "&amp;");
 
 			rssurl = utils::replace_all(rssurl, "\"", "&quot;");
 			link = utils::replace_all(link, "\"", "&quot;");
@@ -754,28 +826,28 @@ void controller::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
 
 
 
-std::vector<rss_item> controller::search_for_items(const std::string& query, const std::string& feedurl) {
-	std::vector<rss_item> items = rsscache->search_for_items(query, feedurl);
+std::vector<std::tr1::shared_ptr<rss_item> > controller::search_for_items(const std::string& query, const std::string& feedurl) {
+	std::vector<std::tr1::shared_ptr<rss_item> > items = rsscache->search_for_items(query, feedurl);
 	GetLogger().log(LOG_DEBUG, "controller::search_for_items: setting feed pointers");
-	for (std::vector<rss_item>::iterator it=items.begin();it!=items.end();++it) {
-		it->set_feedptr(get_feed_by_url(it->feedurl()));
+	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=items.begin();it!=items.end();++it) {
+		(*it)->set_feedptr(get_feed_by_url((*it)->feedurl()));
 	}
 	return items;
 }
 
-rss_feed * controller::get_feed_by_url(const std::string& feedurl) {
-	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
-		if (feedurl == it->rssurl())
-			return &(*it);
+std::tr1::shared_ptr<rss_feed> controller::get_feed_by_url(const std::string& feedurl) {
+	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it=feeds.begin();it!=feeds.end();++it) {
+		if (feedurl == (*it)->rssurl())
+			return *it;
 	}
-	return NULL; // shouldn't happen
+	return std::tr1::shared_ptr<rss_feed>();
 }
 
 bool controller::is_valid_podcast_type(const std::string& /* mimetype */) {
 	return true;
 }
 
-void controller::enqueue_url(const std::string& url) {
+void controller::enqueue_url(const std::string& url, std::tr1::shared_ptr<rss_feed> feed) {
 	bool url_found = false;
 	std::fstream f;
 	f.open(queue_file.c_str(), std::fstream::in);
@@ -784,7 +856,8 @@ void controller::enqueue_url(const std::string& url) {
 			std::string line;
 			getline(f, line);
 			if (!f.eof() && line.length() > 0) {
-				if (line == url) {
+				std::vector<std::string> fields = utils::tokenize_quoted(line);
+				if (fields.size() > 0 && fields[0] == url) {
 					url_found = true;
 					break;
 				}
@@ -794,29 +867,30 @@ void controller::enqueue_url(const std::string& url) {
 	}
 	if (!url_found) {
 		f.open(queue_file.c_str(), std::fstream::app | std::fstream::out);
-		f << url << std::endl;
+		std::string filename = generate_enqueue_filename(url, feed);
+		f << url << " " << stfl::quote(filename) << std::endl;
 		f.close();
 	}
 }
 
 void controller::reload_urls_file() {
 	urlcfg->reload();
-	std::vector<rss_feed> new_feeds;
+	std::vector<std::tr1::shared_ptr<rss_feed> > new_feeds;
 
 	for (std::vector<std::string>::const_iterator it=urlcfg->get_urls().begin();it!=urlcfg->get_urls().end();++it) {
 		bool found = false;
-		for (std::vector<rss_feed>::iterator jt=feeds.begin();jt!=feeds.end();++jt) {
-			if (*it == jt->rssurl()) {
+		for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator jt=feeds.begin();jt!=feeds.end();++jt) {
+			if (*it == (*jt)->rssurl()) {
 				found = true;
-				jt->set_tags(urlcfg->get_tags(*it));
+				(*jt)->set_tags(urlcfg->get_tags(*it));
 				new_feeds.push_back(*jt);
 				break;
 			}
 		}
 		if (!found) {
-			rss_feed new_feed(rsscache);
-			new_feed.set_rssurl(*it);
-			new_feed.set_tags(urlcfg->get_tags(*it));
+			std::tr1::shared_ptr<rss_feed> new_feed(new rss_feed(rsscache));
+			new_feed->set_rssurl(*it);
+			new_feed->set_tags(urlcfg->get_tags(*it));
 			try {
 				rsscache->internalize_rssfeed(new_feed);
 			} catch(const dbexception& e) {
@@ -828,12 +902,32 @@ void controller::reload_urls_file() {
 
 	feeds = new_feeds;
 
+	sort_feeds();
+
 	update_feedlist();
 }
 
-void controller::set_feedptrs(rss_feed& feed) {
-	for (std::vector<rss_item>::iterator it=feed.items().begin();it!=feed.items().end();++it) {
-		it->set_feedptr(&feed);
+void controller::edit_urls_file() {
+	const char * editor = getenv("EDITOR");
+	if (!editor)
+		editor = "vi";
+
+	std::string cmdline = utils::strprintf("%s \"%s\"", editor, utils::replace_all(url_file,"\"","\\\"").c_str());
+
+	v->push_empty_formaction();
+	stfl::reset();
+
+	GetLogger().log(LOG_DEBUG, "controller::edit_urls_file: running `%s'", cmdline.c_str());
+	::system(cmdline.c_str());
+
+	v->pop_current_formaction();
+
+	reload_urls_file();
+}
+
+void controller::set_feedptrs(std::tr1::shared_ptr<rss_feed> feed) {
+	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin();it!=feed->items().end();++it) {
+		(*it)->set_feedptr(feed);
 	}
 }
 
@@ -869,24 +963,24 @@ void controller::execute_commands(char ** argv, unsigned int i) {
 	}
 }
 
-void controller::write_item(const rss_item& item, const std::string& filename) {
+void controller::write_item(std::tr1::shared_ptr<rss_item> item, const std::string& filename) {
 	std::vector<std::string> lines;
 	std::vector<linkpair> links; // not used
 	
 	std::string title(_("Title: "));
-	title.append(item.title());
+	title.append(item->title());
 	lines.push_back(title);
 	
 	std::string author(_("Author: "));
-	author.append(item.author());
+	author.append(item->author());
 	lines.push_back(author);
 	
 	std::string date(_("Date: "));
-	date.append(item.pubDate());
+	date.append(item->pubDate());
 	lines.push_back(date);
 
 	std::string link(_("Link: "));
-	link.append(item.link());
+	link.append(item->link());
 	lines.push_back(link);
 	
 	lines.push_back(std::string(""));
@@ -895,7 +989,7 @@ void controller::write_item(const rss_item& item, const std::string& filename) {
 	if (width == 0)
 		width = 80;
 	htmlrenderer rnd(width);
-	rnd.render(item.description(), lines, links, item.feedurl());
+	rnd.render(item->description(), lines, links, item->feedurl());
 
 	std::fstream f;
 	f.open(filename.c_str(),std::fstream::out);
@@ -910,3 +1004,135 @@ void controller::write_item(const rss_item& item, const std::string& filename) {
 void controller::mark_deleted(const std::string& guid, bool b) {
 	rsscache->mark_item_deleted(guid, b);
 }
+
+std::string controller::prepare_message(unsigned int pos, unsigned int max) {
+	if (max > 0) {
+		return utils::strprintf("(%u/%u) ", pos, max);
+	}
+	return "";
+}
+
+void controller::save_feed(std::tr1::shared_ptr<rss_feed> feed, unsigned int pos) {
+	if (!feed->is_empty()) {
+		GetLogger().log(LOG_DEBUG, "controller::reload: feed is nonempty, saving");
+		rsscache->externalize_rssfeed(feed, ign.matches_resetunread(feed->rssurl()));
+		GetLogger().log(LOG_DEBUG, "controller::reload: after externalize_rssfeed");
+
+		rsscache->internalize_rssfeed(feed);
+		GetLogger().log(LOG_DEBUG, "controller::reload: after internalize_rssfeed");
+		feed->set_tags(urlcfg->get_tags(feed->rssurl()));
+		feeds[pos]->items().clear();
+		feeds[pos] = feed;
+		v->notify_itemlist_change(feeds[pos]);
+	} else {
+		GetLogger().log(LOG_DEBUG, "controller::reload: feed is empty, not saving");
+	}
+}
+
+void controller::enqueue_items(std::tr1::shared_ptr<rss_feed> feed) {
+	if (!cfg->get_configvalue_as_bool("podcast-auto-enqueue"))
+		return;
+	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin();it!=feed->items().end();++it) {
+		if (!(*it)->enqueued() && (*it)->enclosure_url().length() > 0) {
+			GetLogger().log(LOG_DEBUG, "controller::reload: enclosure_url = `%s' enclosure_type = `%s'", (*it)->enclosure_url().c_str(), (*it)->enclosure_type().c_str());
+			if (is_valid_podcast_type((*it)->enclosure_type())) {
+				GetLogger().log(LOG_INFO, "controller::reload: enqueuing `%s'", (*it)->enclosure_url().c_str());
+				enqueue_url((*it)->enclosure_url(), feed);
+				(*it)->set_enqueued(true);
+				rsscache->update_rssitem_unread_and_enqueued(*it, feed->rssurl());
+			}
+		}
+	}
+}
+
+std::string controller::generate_enqueue_filename(const std::string& url, std::tr1::shared_ptr<rss_feed> feed) {
+	std::string dlformat = cfg->get_configvalue("download-path");
+	if (dlformat[dlformat.length()-1] != NEWSBEUTER_PATH_SEP[0])
+		dlformat.append(NEWSBEUTER_PATH_SEP);
+
+	fmtstr_formatter fmt;
+	fmt.register_fmt('n', feed->title());
+	fmt.register_fmt('h', get_hostname_from_url(url));
+
+	std::string dlpath = fmt.do_format(dlformat);
+
+	char buf[1024];
+	snprintf(buf, sizeof(buf), "%s", url.c_str());
+	char * base = basename(buf);
+	if (!base || strlen(base) == 0) {
+		char lbuf[128];
+		time_t t = time(NULL);
+		strftime(lbuf, sizeof(lbuf), "%Y-%b-%d-%H%M%S.unknown", localtime(&t));
+		dlpath.append(lbuf);
+	} else {
+		dlpath.append(base);
+	}
+	return dlpath;
+}
+
+std::string controller::get_hostname_from_url(const std::string& url) {
+	std::string hostname;
+	const char * begin = url.c_str();
+	if (url.substr(0,7) == "http://") {
+		begin = url.c_str() + 7;
+	} else if (url.substr(0,8) == "https://") {
+		begin = url.c_str() + 8;
+	}
+	const char * slash = strchr(begin, '/');
+	if (!slash)
+		hostname = std::string(begin);
+	else
+		hostname = std::string(begin, slash-begin);
+
+	return hostname;
+}
+
+void controller::import_read_information(const std::string& readinfofile) {
+	std::vector<std::string> guids;
+
+	std::ifstream f(readinfofile.c_str());
+	std::string line;
+	getline(f,line);
+	if (!f.is_open()) {
+		return;
+	}
+	while (f.is_open() && !f.eof()) {
+		guids.push_back(line);
+		getline(f, line);
+	}
+	rsscache->mark_items_read_by_guid(guids);
+}
+
+void controller::export_read_information(const std::string& readinfofile) {
+	std::vector<std::string> guids = rsscache->get_read_item_guids();
+
+	std::fstream f;
+	f.open(readinfofile.c_str(), std::fstream::out);
+	if (f.is_open()) {
+		for (std::vector<std::string>::iterator it=guids.begin();it!=guids.end();it++) {
+			f << *it << std::endl;
+		}
+	}
+}
+
+struct sort_feeds_by_firsttag : public std::binary_function<const std::tr1::shared_ptr<rss_feed>&, const std::tr1::shared_ptr<rss_feed>&, bool> {
+	sort_feeds_by_firsttag() { }
+	bool operator()(const std::tr1::shared_ptr<rss_feed>& a, const std::tr1::shared_ptr<rss_feed>& b) {
+		if (a->get_firsttag().length() == 0 || b->get_firsttag().length() == 0) {
+			return a->get_firsttag().length() > b->get_firsttag().length();
+		}
+		return strcasecmp(a->get_firsttag().c_str(), b->get_firsttag().c_str()) < 0;
+	}
+};
+
+
+void controller::sort_feeds() {
+	std::string sortmethod = cfg->get_configvalue("feed-sort-order");
+	if (sortmethod == "none") {
+		// that's the default, do nothing
+	} else if (sortmethod == "firsttag") {
+		std::stable_sort(feeds.begin(), feeds.end(), sort_feeds_by_firsttag());
+	}
+}
+
+}
