@@ -21,8 +21,8 @@ namespace newsbeuter {
 
 feedlist_formaction::feedlist_formaction(view * vv, std::string formstr) 
 	: formaction(vv,formstr), zero_feedpos(false), feeds_shown(0),
-		auto_open(false), quit(false), apply_filter(false), search_dummy_feed(v->get_ctrl()->get_cache()),
-		filterpos(0), set_filterpos(false) {
+		auto_open(false), quit(false), apply_filter(false), search_dummy_feed(new rss_feed(v->get_ctrl()->get_cache())),
+		filterpos(0), set_filterpos(false), rxman(0), old_width(0) {
 	assert(true==m.parse(FILTER_UNREAD_FEEDS));
 }
 
@@ -55,12 +55,7 @@ void feedlist_formaction::init() {
 feedlist_formaction::~feedlist_formaction() { }
 
 void feedlist_formaction::prepare() {
-	static unsigned int old_width = 0;
-
-	std::string listwidth = f->get("items:w");
-	std::istringstream is(listwidth);
-	unsigned int width = 0;
-	is >> width;
+	unsigned int width = utils::to_u(f->get("items:w"));
 
 	if (old_width != width) {
 		do_redraw = true;
@@ -72,20 +67,7 @@ void feedlist_formaction::prepare() {
 		GetLogger().log(LOG_DEBUG, "feedlist_formaction::prepare: doing redraw");
 		do_redraw = false;
 		v->get_ctrl()->update_feedlist();
-		if (set_filterpos) {
-			set_filterpos = false;
-			unsigned int i = 0;
-			for (std::vector<feedptr_pos_pair>::iterator it=visible_feeds.begin();it!=visible_feeds.end();++it, ++i) {
-				if (it->second == filterpos) {
-					f->set("feedpos", utils::to_s(i));
-					return;
-				}
-			}
-			f->set("feedpos", "0");
-		} else if (zero_feedpos) {
-			f->set("feedpos","0");
-			zero_feedpos = false;
-		}
+		set_pos();
 	}
 }
 
@@ -126,7 +108,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			v->get_ctrl()->reload_urls_file();
 			break;
 		case OP_OPENINBROWSER: {
-				rss_feed * feed = v->get_ctrl()->get_feed(pos);
+				std::tr1::shared_ptr<rss_feed> feed = v->get_ctrl()->get_feed(pos);
 				if (feed) {
 					v->open_in_browser(feed->link());
 				}
@@ -173,17 +155,17 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			do_redraw = true;
 			break;
 		case OP_NEXTUNREAD: {
-				unsigned int feedpos;
+				unsigned int local_tmp;
 				GetLogger().log(LOG_INFO, "feedlist_formaction: jumping to next unred feed");
-				if (!jump_to_next_unread_feed(feedpos)) {
+				if (!jump_to_next_unread_feed(local_tmp)) {
 					v->show_error(_("No feeds with unread items."));
 				}
 			}
 			break;
 		case OP_PREVUNREAD: {
-				unsigned int feedpos;
+				unsigned int local_tmp;
 				GetLogger().log(LOG_INFO, "feedlist_formaction: jumping to previous unred feed");
-				if (!jump_to_previous_unread_feed(feedpos)) {
+				if (!jump_to_previous_unread_feed(local_tmp)) {
 					v->show_error(_("No feeds with unread items."));
 				}
 			}
@@ -244,7 +226,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_SEARCH:
 			if (automatic && args->size() > 0) {
-				qna_responses.erase(qna_responses.begin(), qna_responses.end());
+				qna_responses.clear();
 				// when in automatic mode, we manually fill the qna_responses vector from the arguments
 				// and then run the finished_qna() by ourselves to simulate a "Q&A" session that is
 				// in fact macro-driven.
@@ -264,7 +246,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_SETFILTER:
 			if (automatic && args->size() > 0) {
-				qna_responses.erase(qna_responses.begin(), qna_responses.end());
+				qna_responses.clear();
 				qna_responses.push_back((*args)[0]);
 				finished_qna(OP_INT_END_SETFILTER);
 			} else {
@@ -273,6 +255,9 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 				this->start_qna(qna, OP_INT_END_SETFILTER, &filterhistory);
 			}
 			break;
+		case OP_EDIT_URLS:
+			v->get_ctrl()->edit_urls_file();
+			break;
 		case OP_QUIT:
 			GetLogger().log(LOG_INFO, "feedlist_formaction: quitting");
 			if (automatic || !v->get_cfg()->get_configvalue_as_bool("confirm-exit") || v->confirm(_("Do you really want to quit (y:Yes n:No)? "), _("yn")) == *_("y")) {
@@ -290,33 +275,28 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 	}
 }
 
-void feedlist_formaction::update_visible_feeds(std::vector<rss_feed>& feeds) {
+void feedlist_formaction::update_visible_feeds(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds) {
 	assert(v->get_cfg() != NULL); // must not happen
 
-	if (visible_feeds.size() > 0)
-		visible_feeds.erase(visible_feeds.begin(), visible_feeds.end());
+	visible_feeds.clear();
 
 	unsigned int i = 0;
 
-	for (std::vector<rss_feed>::iterator it = feeds.begin(); it != feeds.end(); ++it, ++i) {
-		if ((tag == "" || it->matches_tag(tag)) && (!apply_filter || m.matches(&(*it)))) {
-			visible_feeds.push_back(feedptr_pos_pair(&(*it),i));
+	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it = feeds.begin(); it != feeds.end(); ++it, ++i) {
+		if ((tag == "" || (*it)->matches_tag(tag)) && (!apply_filter || m.matches(it->get()))) {
+			visible_feeds.push_back(feedptr_pos_pair(*it,i));
 		}
 	}
 
 	feeds_shown = visible_feeds.size();
 }
 
-void feedlist_formaction::set_feedlist(std::vector<rss_feed>& feeds) {
+void feedlist_formaction::set_feedlist(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds) {
 	assert(v->get_cfg() != NULL); // must not happen
 
-	std::string listwidth = f->get("feeds:w");
-	std::istringstream is(listwidth);
-	unsigned int width;
-	is >> width;
+	unsigned int width = utils::to_u(f->get("feeds:w"));
 
 	unsigned int i = 0;
-	unsigned short feedlist_number = 1;
 	unsigned int unread_feeds = 0;
 
 	std::string feedlist_format = v->get_cfg()->get_configvalue("feedlist-format");
@@ -325,39 +305,16 @@ void feedlist_formaction::set_feedlist(std::vector<rss_feed>& feeds) {
 
 	update_visible_feeds(feeds);
 
-	for (std::vector<feedptr_pos_pair>::iterator it = visible_feeds.begin(); it != visible_feeds.end(); ++it, ++i, ++feedlist_number) {
-		std::string title = it->first->title();
-		if (title.length()==0) {
-			title = it->first->rssurl(); // rssurl must always be present.
-			if (title.length()==0) {
-				title = "<no title>"; // shouldn't happen
-			}
-		}
+	for (std::vector<feedptr_pos_pair>::iterator it = visible_feeds.begin(); it != visible_feeds.end(); ++it, ++i) {
+		std::string title = get_title(it->first);
 
-		unsigned int unread_count = 0;
-		if (it->first->items().size() > 0) {
-			unread_count = it->first->unread_item_count();
-		}
-		if (unread_count > 0)
+		if (it->first->unread_item_count() > 0)
 			++unread_feeds;
 
-		fmtstr_formatter fmt;
-
-		fmt.register_fmt('i', utils::strprintf("%u", it->second + 1));
-		fmt.register_fmt('u', utils::strprintf("(%u/%u)",unread_count,static_cast<unsigned int>(it->first->items().size())));
-		fmt.register_fmt('n', unread_count > 0 ? "N" : " ");
-		fmt.register_fmt('t', title);
-		fmt.register_fmt('l', it->first->link());
-		fmt.register_fmt('L', it->first->rssurl());
-		fmt.register_fmt('d', it->first->description());
-
-		std::string format = fmt.do_format(feedlist_format, width);
-		GetLogger().log(LOG_DEBUG, "feedlist_formaction::set_feedlist: format result = %s", format.c_str());
-
-		listfmt.add_line(format, it->second);
+		listfmt.add_line(format_line(feedlist_format, it->first, it->second, width), it->second);
 	}
 
-	f->modify("feeds","replace_inner",listfmt.format_list());
+	f->modify("feeds","replace_inner",listfmt.format_list(rxman, "feedlist"));
 
 	std::string title_format = v->get_cfg()->get_configvalue("feedlist-title-format");
 
@@ -464,7 +421,7 @@ bool feedlist_formaction::jump_to_next_unread_feed(unsigned int& feedpos) {
 	return false;
 }
 
-rss_feed * feedlist_formaction::get_feed() {
+std::tr1::shared_ptr<rss_feed> feedlist_formaction::get_feed() {
 	unsigned int curpos;
 	std::istringstream is(f->get("feedpos"));
 	is >> curpos;
@@ -489,16 +446,7 @@ void feedlist_formaction::handle_cmdline(const std::string& cmd) {
 	 * handle it as a normal command.
 	 */
 	if (1==sscanf(cmd.c_str(),"%u",&idx)) {
-		if (idx > 0 && idx <= (visible_feeds[visible_feeds.size()-1].second + 1)) {
-			int i = get_pos(idx - 1);
-			if (i == -1) {
-				v->show_error(_("Position not visible!"));
-			} else {
-				f->set("feedpos", utils::to_s(i));
-			}
-		} else {
-			v->show_error(_("Invalid position!"));
-		}
+		handle_cmdline_num(idx);
 	} else {
 		// hand over all other commands to formaction
 		std::vector<std::string> tokens = utils::tokenize_quoted(cmd, " \t");
@@ -524,42 +472,11 @@ void feedlist_formaction::finished_qna(operation op) {
 	formaction::finished_qna(op); // important!
 
 	switch (op) {
-		case OP_INT_END_SETFILTER: {
-				std::string filtertext = qna_responses[0];
-				filterhistory.add_line(filtertext);
-				if (filtertext.length() > 0) {
-					if (!m.parse(filtertext)) {
-						v->show_error(_("Error: couldn't parse filter command!"));
-						m.parse(FILTER_UNREAD_FEEDS);
-					} else {
-						save_filterpos();
-						apply_filter = true;
-						do_redraw = true;
-					}
-				}
-			}
+		case OP_INT_END_SETFILTER:
+			op_end_setfilter();
 			break;
-		case OP_INT_START_SEARCH: {
-				std::string searchphrase = qna_responses[0];
-				if (searchphrase.length() > 0) {
-					v->set_status(_("Searching..."));
-					searchhistory.add_line(searchphrase);
-					std::vector<rss_item> items;
-					try {
-						std::string utf8searchphrase = utils::convert_text(searchphrase, "utf-8", nl_langinfo(CODESET));
-						items = v->get_ctrl()->search_for_items(utf8searchphrase, "");
-					} catch (const dbexception& e) {
-						v->show_error(utils::strprintf(_("Error while searching for `%s': %s"), searchphrase.c_str(), e.what()));
-						return;
-					}
-					if (items.size() > 0) {
-						search_dummy_feed.items() = items;
-						v->push_searchresult(&search_dummy_feed);
-					} else {
-						v->show_error(_("No results."));
-					}
-				}
-			}
+		case OP_INT_START_SEARCH:
+			op_start_search();
 			break;
 		default:
 			break;
@@ -589,4 +506,115 @@ void feedlist_formaction::save_filterpos() {
 	}
 }
 
+void feedlist_formaction::set_regexmanager(regexmanager * r) {
+	rxman = r;
+	std::vector<std::string>& attrs = r->get_attrs("feedlist");
+	unsigned int i=0;
+	std::string attrstr;
+	for (std::vector<std::string>::iterator it=attrs.begin();it!=attrs.end();++it,++i) {
+		attrstr.append(utils::strprintf("@style_%u_normal:%s ", i, it->c_str()));
+		attrstr.append(utils::strprintf("@style_%u_focus:%s ", i, it->c_str()));
+	}
+	std::string textview = utils::strprintf("{!list[feeds] .expand:vh style_normal[listnormal]: style_focus[listfocus]:fg=yellow,bg=blue,attr=bold pos_name[feedposname]: pos[feedpos]:0 %s richtext:1}", attrstr.c_str());
+	f->modify("feeds", "replace", textview);
+}
+
+
+void feedlist_formaction::op_end_setfilter() {
+	std::string filtertext = qna_responses[0];
+	filterhistory.add_line(filtertext);
+	if (filtertext.length() > 0) {
+		if (!m.parse(filtertext)) {
+			v->show_error(_("Error: couldn't parse filter command!"));
+			m.parse(FILTER_UNREAD_FEEDS);
+		} else {
+			save_filterpos();
+			apply_filter = true;
+			do_redraw = true;
+		}
+	}
+}
+
+
+void feedlist_formaction::op_start_search() {
+	std::string searchphrase = qna_responses[0];
+	GetLogger().log(LOG_DEBUG, "feedlist_formaction::op_start_search: starting search for `%s'", searchphrase.c_str());
+	if (searchphrase.length() > 0) {
+		v->set_status(_("Searching..."));
+		searchhistory.add_line(searchphrase);
+		std::vector<std::tr1::shared_ptr<rss_item> > items;
+		try {
+			std::string utf8searchphrase = utils::convert_text(searchphrase, "utf-8", nl_langinfo(CODESET));
+			items = v->get_ctrl()->search_for_items(utf8searchphrase, "");
+		} catch (const dbexception& e) {
+			v->show_error(utils::strprintf(_("Error while searching for `%s': %s"), searchphrase.c_str(), e.what()));
+			return;
+		}
+		if (items.size() > 0) {
+			search_dummy_feed->items() = items;
+			v->push_searchresult(search_dummy_feed);
+		} else {
+			v->show_error(_("No results."));
+		}
+	}
+}
+
+void feedlist_formaction::handle_cmdline_num(unsigned int idx) {
+	if (idx > 0 && idx <= (visible_feeds[visible_feeds.size()-1].second + 1)) {
+		int i = get_pos(idx - 1);
+		if (i == -1) {
+			v->show_error(_("Position not visible!"));
+		} else {
+			f->set("feedpos", utils::to_s(i));
+		}
+	} else {
+		v->show_error(_("Invalid position!"));
+	}
+}
+
+void feedlist_formaction::set_pos() {
+	if (set_filterpos) {
+		set_filterpos = false;
+		unsigned int i = 0;
+		for (std::vector<feedptr_pos_pair>::iterator it=visible_feeds.begin();it!=visible_feeds.end();++it, ++i) {
+			if (it->second == filterpos) {
+				f->set("feedpos", utils::to_s(i));
+				return;
+			}
+		}
+		f->set("feedpos", "0");
+	} else if (zero_feedpos) {
+		f->set("feedpos","0");
+		zero_feedpos = false;
+	}
+}
+
+std::string feedlist_formaction::get_title(std::tr1::shared_ptr<rss_feed> feed) {
+	std::string title = feed->title();
+	if (title.length()==0)
+		title = feed->rssurl();
+	if (title.length()==0)
+		title = "<no title>";
+	return title;
+}
+
+std::string feedlist_formaction::format_line(const std::string& feedlist_format, std::tr1::shared_ptr<rss_feed> feed, unsigned int pos, unsigned int width) {
+	fmtstr_formatter fmt;
+	unsigned int unread_count = feed->unread_item_count();
+
+	fmt.register_fmt('i', utils::strprintf("%u", pos + 1));
+	fmt.register_fmt('u', utils::strprintf("(%u/%u)",unread_count,static_cast<unsigned int>(feed->items().size())));
+	fmt.register_fmt('n', unread_count > 0 ? "N" : " ");
+	fmt.register_fmt('t', get_title(feed));
+	fmt.register_fmt('T', feed->get_firsttag());
+	fmt.register_fmt('l', feed->link());
+	fmt.register_fmt('L', feed->rssurl());
+	fmt.register_fmt('d', feed->description());
+
+	std::string format = fmt.do_format(feedlist_format, width);
+	GetLogger().log(LOG_DEBUG, "feedlist_formaction::set_feedlist: format result = %s", format.c_str());
+
+	return format;
+}
+
 }
