@@ -39,7 +39,7 @@ std::string formaction::prepare_keymap_hint(keymap_hint_entry * hints) {
 	 */
 	std::string keymap_hint;
 	for (int i=0;hints[i].op != OP_NIL; ++i) {
-		keymap_hint.append(v->get_keys()->getkey(hints[i].op));
+		keymap_hint.append(v->get_keys()->getkey(hints[i].op, this->id()));
 		keymap_hint.append(":");
 		keymap_hint.append(hints[i].text);
 		keymap_hint.append(" ");
@@ -64,6 +64,20 @@ void formaction::process_op(operation op, bool automatic, std::vector<std::strin
 		case OP_CMDLINE: 
 			start_cmdline();
 			break;
+		case OP_INT_SET:
+			if (automatic) {
+				std::string cmdline = "set ";
+				if (args) {
+					for (std::vector<std::string>::iterator it=args->begin();it!=args->end();it++) {
+						cmdline.append(utils::strprintf("%s ", stfl::quote(*it).c_str()));
+					}
+				}
+				GetLogger().log(LOG_DEBUG, "formaction::process_op: running commandline `%s'", cmdline.c_str());
+				this->handle_cmdline(cmdline);
+			} else {
+				GetLogger().log(LOG_WARN, "formaction::process_op: got OP_INT_SET, but not automatic");
+			}
+			break;
 		case OP_INT_CANCEL_QNA:
 			f->modify("lastline","replace","{hbox[lastline] .expand:0 {label[msglabel] .expand:h text[msg]:\"\"}}");
 			break;
@@ -136,9 +150,7 @@ void formaction::start_qna(const std::vector<qna_pair>& prompts, operation finis
 	 * vector, the first fields corresponds with the first prompt, the second field with the second prompt, etc.
 	 */
 	qna_prompts = prompts;
-	if (qna_responses.size() > 0) {
-		qna_responses.erase(qna_responses.begin(), qna_responses.end());
-	}
+	qna_responses.clear();
 	finish_operation = finish_op;
 	qna_history = h;
 	start_next_question();
