@@ -62,6 +62,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 	bool itunes_hack = false;
 	unsigned int ol_count = 1;
 	htmltag current_tag;
+	int link_num = -1;
 	
 	/*
 	 * to render the HTML, we use a self-developed "XML" pull parser.
@@ -93,8 +94,8 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 								link = "";
 							}
 							if (link.length() > 0) {
-								unsigned int link_num = add_link(links,utils::absolute_url(url,link), LINK_HREF);
-								curline.append(utils::strprintf("[%u]", link_num));
+								link_num = add_link(links,utils::absolute_url(url,link), LINK_HREF);
+								curline.append("[");
 							}
 						}
 						break;
@@ -329,6 +330,12 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 						break;
 
 					case TAG_A:
+						if (link_num != -1) {
+							curline.append(utils::strprintf("][%d]", link_num));
+							link_num = -1;
+						}
+						break;
+
 					case TAG_EMBED:
 					case TAG_BR:
 					case TAG_ITUNESHACK:
@@ -389,6 +396,11 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 
 						unsigned int i=0;
 						bool new_line = false;
+
+						if (!line_is_nonempty(curline) && words.size() > 0 && words[0] == " ") {
+							words.erase(words.begin());
+						}
+
 						GetLogger().log(LOG_DEBUG, "htmlrenderer::render: tokenized `%s'", xpp.getText().c_str());
 						for (std::vector<std::string>::iterator it=words.begin();it!=words.end();++it,++i) {
 							GetLogger().log(LOG_DEBUG, "htmlrenderer::render: token[%u] = `%s'", i, it->c_str());
