@@ -12,7 +12,7 @@
 namespace newsbeuter {
 
 itemview_formaction::itemview_formaction(view * vv, std::string formstr)
-	: formaction(vv,formstr), feed(0), show_source(false), quit(false) { 
+	: formaction(vv,formstr), show_source(false), quit(false), rxman(0), num_lines(0) { 
 }
 
 itemview_formaction::~itemview_formaction() { }
@@ -22,13 +22,18 @@ void itemview_formaction::init() {
 	f->set("msg","");
 	do_redraw = true;
 	quit = false;
-	links.erase(links.begin(), links.end());
+	links.clear();
+	num_lines = 0;
+	if (!v->get_cfg()->get_configvalue_as_bool("display-article-progress")) {
+		f->set("percentwidth", "0");
+	} else {
+		f->set("percentwidth", utils::to_s(utils::max(6, utils::max(strlen(_("Top")), strlen(_("Bottom"))))));
+		update_percent();
+	}
 	set_keymap_hints();
 }
 
 void itemview_formaction::prepare() {
-	static bool render_hack;
-
 	/*
 	 * whenever necessary, the item view is regenerated. This is done
 	 * by putting together the feed name, title, link, author, optional
@@ -36,83 +41,90 @@ void itemview_formaction::prepare() {
 	 * HTML. The links extracted by the renderer are then appended, too.
 	 */
 	if (do_redraw) {
-		rss_item& item = feed->get_item_by_guid(guid);
-		listformatter listfmt;
 
-		rss_feed * feedptr = item.get_feedptr();
-
-		std::string title, feedtitle;
-		if (feedptr->title().length() > 0) {
-			title = feedptr->title();
-		} else if (feedptr->link().length() > 0) {
-			title = feedptr->link();
-		} else if (feedptr->rssurl().length() > 0) {
-			title = feedptr->rssurl();
-		}
-		feedtitle = utils::strprintf("%s%s", _("Feed: "), title.c_str());
-		listfmt.add_line(feedtitle);
-
-		if (item.title().length() > 0) {
-			title = utils::strprintf("%s%s", _("Title: "), item.title().c_str());
-			listfmt.add_line(title);
-		}
-
-		if (item.author().length() > 0) {
-			std::string author = utils::strprintf("%s%s", _("Author: "), item.author().c_str());
-			listfmt.add_line(author);
-		}
-
-		if (item.link().length() > 0) {
-			std::string link = utils::strprintf("%s%s", _("Link: "), item.link().c_str());
-			listfmt.add_line(link);
-		}
-
-		std::string date = utils::strprintf("%s%s", _("Date: "), item.pubDate().c_str());
-		listfmt.add_line(date);
-
-		if (item.flags().length() > 0) {
-			std::string flags = utils::strprintf("%s%s", _("Flags: "), item.flags().c_str());
-			listfmt.add_line(flags);
-		}
-
-		if (item.enclosure_url().length() > 0) {
-			std::string enc_url = utils::strprintf("%s%s (%s%s)", _("Podcast Download URL: "), item.enclosure_url().c_str(), _("type: "), item.enclosure_type().c_str());
-			listfmt.add_line(enc_url);
-		}
-
-		listfmt.add_line("");
-
-		set_head(item.title());
-
-		if (!render_hack) {
-			f->run(-3); // XXX HACK: render once so that we get a proper widget width
-			render_hack = true;
+		{
+		scope_measure("itemview::prepare: rendering");
+		f->run(-3); // XXX HACK: render once so that we get a proper widget width
 		}
 
 		std::vector<std::string> lines;
 		std::string widthstr = f->get("article:w");
 		unsigned int render_width = 80;
+		unsigned int view_width;
 		if (widthstr.length() > 0) {
-			std::istringstream is(widthstr);
-			is >> render_width;
+			view_width = render_width = utils::to_u(widthstr);
 			if (render_width - 5 > 0)
 				render_width -= 5; 	
 		}
 
+		std::tr1::shared_ptr<rss_item> item = feed->get_item_by_guid(guid);
+		listformatter listfmt;
+
+		std::tr1::shared_ptr<rss_feed> feedptr = item->get_feedptr();
+
+		std::string title, feedtitle;
+		if (feedptr.get() != NULL) {
+			if (feedptr->title().length() > 0) {
+				title = feedptr->title();
+			} else if (feedptr->link().length() > 0) {
+				title = feedptr->link();
+			} else if (feedptr->rssurl().length() > 0) {
+				title = feedptr->rssurl();
+			}
+		}
+		if (title.length() > 0) {
+			feedtitle = utils::strprintf("%s%s", _("Feed: "), title.c_str());
+			listfmt.add_line(feedtitle, UINT_MAX, view_width);
+		}
+
+		if (item->title().length() > 0) {
+			title = utils::strprintf("%s%s", _("Title: "), item->title().c_str());
+			listfmt.add_line(title, UINT_MAX, view_width);
+		}
+
+		if (item->author().length() > 0) {
+			std::string author = utils::strprintf("%s%s", _("Author: "), item->author().c_str());
+			listfmt.add_line(author, UINT_MAX, view_width);
+		}
+
+		if (item->link().length() > 0) {
+			std::string link = utils::strprintf("%s%s", _("Link: "), item->link().c_str());
+			listfmt.add_line(link, UINT_MAX, view_width);
+		}
+
+		std::string date = utils::strprintf("%s%s", _("Date: "), item->pubDate().c_str());
+		listfmt.add_line(date, UINT_MAX, view_width);
+
+		if (item->flags().length() > 0) {
+			std::string flags = utils::strprintf("%s%s", _("Flags: "), item->flags().c_str());
+			listfmt.add_line(flags, UINT_MAX, view_width);
+		}
+
+		if (item->enclosure_url().length() > 0) {
+			std::string enc_url = utils::strprintf("%s%s (%s%s)", _("Podcast Download URL: "), item->enclosure_url().c_str(), _("type: "), item->enclosure_type().c_str());
+			listfmt.add_line(enc_url, UINT_MAX, view_width);
+		}
+
+		listfmt.add_line("");
+
+		set_head(item->title());
+
 		unsigned int textwidth = v->get_cfg()->get_configvalue_as_int("text-width");
 		if (textwidth > 0) {
 			render_width = textwidth;
 		}
 
 		if (show_source) {
-			render_source(lines, item.description(), render_width);
+			render_source(lines, item->description(), render_width);
 		} else {
-			lines = render_html(item.description(), links, item.feedurl(), render_width);
+			lines = render_html(item->description(), links, item->feedurl(), render_width);
 		}
 
-		listfmt.add_lines(lines);
+		listfmt.add_lines(lines, view_width);
 
-		f->modify("article","replace_inner",listfmt.format_list());
+		num_lines = listfmt.get_lines_count();
+
+		f->modify("article","replace_inner",listfmt.format_list(rxman, "article"));
 		f->set("articleoffset","0");
 
 		do_redraw = false;
@@ -121,7 +133,7 @@ void itemview_formaction::prepare() {
 }
 
 void itemview_formaction::process_operation(operation op, bool automatic, std::vector<std::string> * args) {
-	rss_item& item = feed->get_item_by_guid(guid);
+	std::tr1::shared_ptr<rss_item> item = feed->get_item_by_guid(guid);
 
 	/*
 	 * whenever we process an operation, we mark the item
@@ -131,7 +143,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 	 * recorded in the database.
 	 */
 	try {
-		item.set_unread(false);
+		item->set_unread(false);
 	} catch (const dbexception& e) {
 		v->show_error(utils::strprintf(_("Error while marking article as read: %s"), e.what()));
 	}
@@ -146,9 +158,9 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			do_redraw = true;
 			break;
 		case OP_ENQUEUE: {
-				if (item.enclosure_url().length() > 0) {
-					v->get_ctrl()->enqueue_url(item.enclosure_url());
-					v->set_status(utils::strprintf(_("Added %s to download queue."), item.enclosure_url().c_str()));
+				if (item->enclosure_url().length() > 0) {
+					v->get_ctrl()->enqueue_url(item->enclosure_url(), feed);
+					v->set_status(utils::strprintf(_("Added %s to download queue."), item->enclosure_url().c_str()));
 				}
 			}
 			break;
@@ -160,7 +172,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 					if (args->size() > 0)
 						filename = (*args)[0];
 				} else {
-					filename = v->run_filebrowser(FBT_SAVE,v->get_filename_suggestion(item.title()));
+					filename = v->run_filebrowser(FBT_SAVE,v->get_filename_suggestion(item->title()));
 				}
 				if (filename == "") {
 					v->show_error(_("Aborted saving."));
@@ -177,29 +189,29 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 		case OP_OPENINBROWSER:
 			GetLogger().log(LOG_INFO, "view::run_itemview: starting browser");
 			v->set_status(_("Starting browser..."));
-			v->open_in_browser(item.link());
+			v->open_in_browser(item->link());
 			v->set_status("");
 			break;
 		case OP_BOOKMARK:
 			if (automatic) {
-				qna_responses.erase(qna_responses.begin(), qna_responses.end());
-				qna_responses.push_back(item.title());
-				qna_responses.push_back(item.link());
+				qna_responses.clear();
+				qna_responses.push_back(item->title());
+				qna_responses.push_back(item->link());
 				qna_responses.push_back(args->size() > 0 ? (*args)[0] : "");
 			} else {
-				this->start_bookmark_qna(item.title(), item.link(), "");
+				this->start_bookmark_qna(item->title(), item->link(), "");
 			}
 			break;
 		case OP_EDITFLAGS: 
 			if (automatic) {
-				qna_responses.erase(qna_responses.begin(), qna_responses.end());
+				qna_responses.clear();
 				if (args->size() > 0) {
 					qna_responses.push_back((*args)[0]);
 					this->finished_qna(OP_INT_EDITFLAGS_END);
 				}
 			} else {
 				std::vector<qna_pair> qna;
-				qna.push_back(qna_pair(_("Flags: "), item.flags()));
+				qna.push_back(qna_pair(_("Flags: "), item->flags()));
 				this->start_qna(qna, OP_INT_EDITFLAGS_END);
 			}
 			break;
@@ -229,6 +241,17 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 				v->show_error(_("No unread items."));
 			}
 			break;
+		case OP_TOGGLEITEMREAD:
+			GetLogger().log(LOG_INFO, "view::run_itemview: setting unread and quitting");
+			v->set_status(_("Toggling read flag for article..."));
+			try {
+				item->set_unread(true);
+			} catch (const dbexception& e) {
+				v->show_error(utils::strprintf(_("Error while marking article as unread: %s"), e.what()));
+			}
+			v->set_status("");
+			quit = true;
+			break;
 		case OP_QUIT:
 			GetLogger().log(LOG_INFO, "view::run_itemview: quitting");
 			quit = true;
@@ -236,6 +259,25 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 		case OP_HELP:
 			v->push_help();
 			break;
+		case OP_1:
+		case OP_2:
+		case OP_3:
+		case OP_4:
+		case OP_5:
+		case OP_6:
+		case OP_7:
+		case OP_8:
+		case OP_9:
+		case OP_0: {
+				unsigned int idx = op - OP_1;
+				GetLogger().log(LOG_DEBUG, "itemview::run: OP_1 = %d op = %d idx = %u", OP_1, op, idx);
+				if(idx < links.size()) {
+					v->set_status(_("Starting browser..."));
+					v->open_in_browser(links[idx].first);
+					v->set_status("");
+				}
+			}
+			break;
 		default:
 			break;
 	}
@@ -312,7 +354,7 @@ void itemview_formaction::handle_cmdline(const std::string& cmd) {
 	if (tokens.size() > 0) {
 		if (tokens[0] == "save" && tokens.size() >= 2) {
 			std::string filename = utils::resolve_tilde(tokens[1]);
-			rss_item& item = feed->get_item_by_guid(guid);
+			std::tr1::shared_ptr<rss_item> item = feed->get_item_by_guid(guid);
 
 			if (filename == "") {
 				v->show_error(_("Aborted saving."));
@@ -334,12 +376,12 @@ void itemview_formaction::handle_cmdline(const std::string& cmd) {
 void itemview_formaction::finished_qna(operation op) {
 	formaction::finished_qna(op); // important!
 
-	rss_item& item = feed->get_item_by_guid(guid);
+	std::tr1::shared_ptr<rss_item> item = feed->get_item_by_guid(guid);
 
 	switch (op) {
 		case OP_INT_EDITFLAGS_END:
-			item.set_flags(qna_responses[0]);
-			item.update_flags();
+			item->set_flags(qna_responses[0]);
+			item->update_flags();
 			v->set_status(_("Flags updated."));
 			do_redraw = true;
 			break;
@@ -348,14 +390,19 @@ void itemview_formaction::finished_qna(operation op) {
 	}
 }
 
-std::vector<std::string> itemview_formaction::render_html(const std::string& source, std::vector<linkpair>& links, const std::string& feedurl, unsigned int render_width) {
+std::vector<std::string> itemview_formaction::render_html(const std::string& source, std::vector<linkpair>& thelinks, const std::string& feedurl, unsigned int render_width) {
 	std::vector<std::string> lines;
 	std::string renderer = v->get_cfg()->get_configvalue("html-renderer");
 	if (renderer == "internal") {
 		htmlrenderer rnd(render_width);
-		rnd.render(source, lines, links, feedurl);
+		rnd.render(source, lines, thelinks, feedurl);
 	} else {
-		std::string output = utils::run_filter(renderer, source);
+		char * argv[4];
+		argv[0] = const_cast<char *>("/bin/sh");
+		argv[1] = const_cast<char *>("-c");
+		argv[2] = const_cast<char *>(renderer.c_str());
+		argv[3] = NULL;
+		std::string output = utils::run_program(argv, source);
 		std::istringstream is(output);
 		std::string line;
 		getline(is, line);
@@ -367,4 +414,40 @@ std::vector<std::string> itemview_formaction::render_html(const std::string& sou
 	return lines;
 }
 
+void itemview_formaction::set_regexmanager(regexmanager * r) {
+	rxman = r;
+	std::vector<std::string>& attrs = r->get_attrs("article");
+	unsigned int i=0;
+	std::string attrstr;
+	for (std::vector<std::string>::iterator it=attrs.begin();it!=attrs.end();++it,++i) {
+		attrstr.append(utils::strprintf("@style_%u_normal:%s ", i, it->c_str()));
+	}
+	std::string textview = utils::strprintf("{textview[article] style_normal[article]: style_end[styleend]:fg=blue,attr=bold %s .expand:vh offset[articleoffset]:0 richtext:1}", attrstr.c_str());
+	f->modify("article", "replace", textview);
+}
+
+void itemview_formaction::update_percent() {
+	if (v->get_cfg()->get_configvalue_as_bool("display-article-progress")) {
+		std::istringstream is(f->get("articleoffset"));
+		unsigned int offset = 0;
+		unsigned int percent = 0;
+		is >> offset;
+
+		if (num_lines > 0)
+			percent = (100 * (offset + 1)) / num_lines;
+		else
+			percent = 0;
+
+		GetLogger().log(LOG_DEBUG, "itemview_formaction::update_percent: offset = %u num_lines = %u percent = %u", offset, num_lines, percent);
+
+		if (offset == 0 || percent == 0) {
+			f->set("percent", _("Top"));
+		} else if (offset == (num_lines - 1)) {
+			f->set("percent", _("Bottom"));
+		} else {
+			f->set("percent", utils::strprintf("%3u %% ", percent));
+		}
+	}
+}
+
 }
