@@ -28,9 +28,6 @@ void ctrl_c_action(int sig) {
 	GetLogger().log(LOG_DEBUG,"caugh signal %d",sig);
 	stfl::reset();
 	utils::remove_fs_lock(lock_file);
-	if (SIGSEGV == sig) {
-		fprintf(stderr,"%s\n", _("Segmentation fault."));
-	}
 	::exit(EXIT_FAILURE);
 }
 
@@ -69,7 +66,6 @@ void pb_controller::run(int argc, char * argv[]) {
 	bool automatic_dl = false;
 
 	::signal(SIGINT, ctrl_c_action);
-	::signal(SIGSEGV, ctrl_c_action);
 
 	do {
 		if ((c = ::getopt(argc, argv, "C:q:d:l:ha")) < 0)
@@ -133,10 +129,10 @@ void pb_controller::run(int argc, char * argv[]) {
 	null_config_action_handler null_cah;
 	cfgparser.register_handler("macro", &null_cah);
 	cfgparser.register_handler("ignore-article", &null_cah);
+	cfgparser.register_handler("always-download", &null_cah);
 	cfgparser.register_handler("define-filter", &null_cah);
-#if HAVE_RUBY
-	cfgparser.register_handler("load", &null_cah);
-#endif
+	cfgparser.register_handler("highlight", &null_cah);
+	cfgparser.register_handler("reset-unread-on-update", &null_cah);
 
 	try {
 		cfgparser.parse("/etc/newsbeuter/config");
@@ -204,9 +200,9 @@ unsigned int pb_controller::get_maxdownloads() {
 	return max_dls;
 }
 
-void pb_controller::reload_queue() {
+void pb_controller::reload_queue(bool remove_unplayed) {
 	if (ql) {
-		ql->reload(downloads_);
+		ql->reload(downloads_, remove_unplayed);
 	}
 }
 
