@@ -3,7 +3,7 @@
 
 using namespace newsbeuter;
 
-reloadthread::reloadthread(controller * c, unsigned int wt_sec, configcontainer * cf) : ctrl(c), oldtime(0), waittime_sec(wt_sec), suppressed_first(false), cfg(cf) {
+reloadthread::reloadthread(controller * c, time_t wt_sec, configcontainer * cf) : ctrl(c), oldtime(0), waittime_sec(wt_sec), suppressed_first(false), cfg(cf) {
 	GetLogger().log(LOG_INFO,"reloadthread: waiting %u seconds between reloads",waittime_sec);
 }
 
@@ -24,7 +24,7 @@ void reloadthread::run() {
 		}
 
 		time_t seconds_to_wait = 0;
-		if (oldtime + waittime_sec > time(NULL))
+		if ((oldtime + waittime_sec) > time(NULL))
 			seconds_to_wait = oldtime + waittime_sec - time(NULL);
 
 		while (seconds_to_wait > 0) {
