@@ -41,17 +41,9 @@ void file_urlreader::reload() {
 	if (offline)
 		return;
 
-	if (urls.size() > 0) {
-		urls.erase(urls.begin(), urls.end());
-	}
-
-	if (tags.size() > 0) {
-		tags.erase(tags.begin(), tags.end());
-	}
-
-	if (alltags.size() > 0) {
-		alltags.erase(alltags.begin(), alltags.end());
-	}
+	urls.clear();
+	tags.clear();
+	alltags.clear();
 
 	std::fstream f;
 	f.open(filename.c_str(),std::fstream::in);
@@ -119,21 +111,13 @@ void opml_urlreader::reload() {
 	if (offline)
 		return;
 
-	if (urls.size() > 0) {
-		urls.erase(urls.begin(), urls.end());
-	}
-
-	if (tags.size() > 0) {
-		tags.erase(tags.begin(), tags.end());
-	}
-
-	if (alltags.size() > 0) {
-		alltags.erase(alltags.begin(), alltags.end());
-	}
+	urls.clear();
+	tags.clear();
+	alltags.clear();
 
 	std::string user_agent = utils::get_useragent(cfg);
 
-	std::string urlcontent = utils::retrieve_url(this->get_source(), user_agent.c_str(), this->get_auth());
+	std::string urlcontent = utils::retrieve_url(this->get_source(), user_agent.c_str(), this->get_auth(), cfg->get_configvalue_as_int("download-timeout"));
 	// GetLogger().log(LOG_DEBUG, "bloglines_urlreader::reload: return OPML content is `%s'", urlcontent.c_str());
 
 	nxml_t *data;
