@@ -81,14 +81,7 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::str
 						}
 					} else {
 						if (str[last_pos-1] == '\\') {
-							switch (str[last_pos]) {
-								case 'n': token.append("\n"); break;
-								case 'r': token.append("\r"); break;
-								case 't': token.append("\t"); break;
-								case '"': token.append("\""); break;
-								case '\\': break;
-								default: token.append(1, str[last_pos]); break;
-							}
+							append_escapes(token, str[last_pos]);
 						} else {
 							token.append(1, str[last_pos]);
 						}
@@ -108,14 +101,7 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::str
 						}
 					} else {
 						if (str[last_pos-1] == '\\') {
-							switch (str[last_pos]) {
-								case 'n': token.append("\n"); break;
-								case 'r': token.append("\r"); break;
-								case 't': token.append("\t"); break;
-								case '"': token.append("\""); break;
-								case '\\': break;
-								default: token.append(1, str[last_pos]); break;
-							}
+							append_escapes(token, str[last_pos]);
 						} else {
 							token.append(1, str[last_pos]);
 						}
@@ -348,7 +334,7 @@ static size_t my_write_data(void *buffer, size_t size, size_t nmemb, void *userp
 	return size * nmemb;
 }
 
-std::string utils::retrieve_url(const std::string& url, const char * user_agent, const char * auth) {
+std::string utils::retrieve_url(const std::string& url, const char * user_agent, const char * auth, int download_timeout) {
 	std::string buf;
 
 	CURL * easyhandle = curl_easy_init();
@@ -360,6 +346,7 @@ std::string utils::retrieve_url(const std::string& url, const char * user_agent,
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEDATA, &buf);
 	curl_easy_setopt(easyhandle, CURLOPT_NOSIGNAL, 1);
 	curl_easy_setopt(easyhandle, CURLOPT_ENCODING, "gzip, deflate");
+	curl_easy_setopt(easyhandle, CURLOPT_TIMEOUT, download_timeout);
 
 	if (auth) {
 		curl_easy_setopt(easyhandle, CURLOPT_USERPWD, auth);
@@ -396,62 +383,19 @@ void utils::run_command(const std::string& cmd, const std::string& input) {
 	}
 }
 
-std::string utils::run_filter(const std::string& cmd, const std::string& input) {
-	std::string buf;
-	int ipipe[2];
-	int opipe[2];
-	pipe(ipipe);
-	pipe(opipe);
-
-	int rc = fork();
-	switch (rc) {
-		case -1: break;
-		case 0: { // child:
-				close(ipipe[1]);
-				close(opipe[0]);
-				dup2(ipipe[0], 0);
-				dup2(opipe[1], 1);
-				GetLogger().log(LOG_DEBUG, "utils::run_filter: ipipe[0] = %d ipipe[1] = %d opipe[0] = %d opipe[1] = %d", ipipe[0], ipipe[1], opipe[0], opipe[1]);
-				execl("/bin/sh", "/bin/sh", "-c", cmd.c_str(), NULL);
-				exit(1);
-			}
-			break;
-		default: {
-				close(ipipe[0]);
-				close(opipe[1]);
-				write(ipipe[1], input.c_str(), input.length());
-				close(ipipe[1]);
-				char cbuf[1024];
-				int rc2;
-				while ((rc2 = read(opipe[0], cbuf, sizeof(cbuf))) > 0) {
-					buf.append(cbuf, rc2);
-				}
-				close(opipe[0]);
-			}
-			break;
-	}
-	return buf;
-}
-
 std::string utils::run_program(char * argv[], const std::string& input) {
 	std::string buf;
-	int ipipe[2];
-	int opipe[2];
-	pipe(ipipe);
-	pipe(opipe);
+	int ipipe[2]; int opipe[2];
+	pipe(ipipe);  pipe(opipe);
 
 	int errfd = ::open("/dev/null", O_WRONLY);
 
-
 	int rc = fork();
 	switch (rc) {
 		case -1: break;
 		case 0: { // child:
-				close(ipipe[1]);
-				close(opipe[0]);
-				dup2(ipipe[0], 0);
-				dup2(opipe[1], 1);
-
+				close(ipipe[1]);   close(opipe[0]);
+				dup2(ipipe[0], 0); dup2(opipe[1], 1);
 				close(2);
 				if (errfd != -1) dup2(errfd, 2);
 
@@ -460,8 +404,7 @@ std::string utils::run_program(char * argv[], const std::string& input) {
 			}
 			break;
 		default: {
-				close(ipipe[0]);
-				close(opipe[1]);
+				close(ipipe[0]); close(opipe[1]);
 				write(ipipe[1], input.c_str(), input.length());
 				close(ipipe[1]);
 				char cbuf[1024];
@@ -608,6 +551,12 @@ std::string utils::get_useragent(configcontainer * cfgcont) {
 	return ua_pref;
 }
 
+unsigned int utils::to_u(const std::string& str) {
+	std::istringstream is(str);
+	unsigned int u;
+	is >> u;
+	return u;
+}
 
 scope_measure::scope_measure(const std::string& func, loglevel ll) : lvl(ll) {
 	funcname = func;
@@ -626,4 +575,52 @@ scope_measure::~scope_measure() {
 	GetLogger().log(LOG_INFO, "scope_measure: function `%s' took %lu.%06lu s", funcname.c_str(), diff / 1000000, diff % 1000000);
 }
 
+void utils::append_escapes(std::string& str, char c) {
+	switch (c) {
+		case 'n': str.append("\n"); break;
+		case 'r': str.append("\r"); break;
+		case 't': str.append("\t"); break;
+		case '"': str.append("\""); break;
+		case '\\': break;
+		default: str.append(1, c); break;
+	}
+}
+
+bool utils::is_valid_color(const std::string& color) {
+	const char * colors[] = { "black", "red", "green", "yellow", "blue", "magenta", "cyan", "white", "default", NULL };
+	for (unsigned int i=0;colors[i];i++) {
+		if (color == colors[i])
+			return true;
+	}
+	return false;
+}
+
+bool utils::is_valid_attribute(const std::string& attrib) {
+	const char * attribs[] = { "standout", "underline", "reverse", "blink", "dim", "bold", "protect", "invis", "default", NULL };
+	for (unsigned int i=0;attribs[i];i++) {
+		if (attrib == attribs[i])
+			return true;
+	}
+	return false;
+}
+
+std::vector<std::pair<unsigned int, unsigned int> > utils::partition_indexes(unsigned int start, unsigned int end, unsigned int parts) {
+	std::vector<std::pair<unsigned int, unsigned int> > partitions;
+	unsigned int count = end - start + 1;
+	unsigned int size = count / parts;
+
+	for (unsigned int i=0;i<parts-1;i++) {
+		partitions.push_back(std::pair<unsigned int, unsigned int>(start, start + size - 1));
+		start += size;
+	}
+
+	partitions.push_back(std::pair<unsigned int, unsigned int>(start, end));
+	return partitions;
+}
+
+size_t utils::strwidth(const std::string& str) {
+	std::wstring wstr = str2wstr(str);
+	return wcswidth(wstr.c_str(), wstr.length());
+}
+
 }
