@@ -5,6 +5,7 @@
 #include <filebrowser.h>
 #include <urlview.h>
 #include <selecttag.h>
+#include <formatstring.h>
 
 #include <formaction.h>
 #include <feedlist_formaction.h>
@@ -20,6 +21,7 @@
 #include <exceptions.h>
 #include <keymap.h>
 #include <utils.h>
+#include <regexmanager.h>
 
 #include <iostream>
 #include <iomanip>
@@ -59,7 +61,6 @@ view::view(controller * c) : ctrl(c), cfg(0), keys(0), mtx(0) {
 	filebrowser = new filebrowser_formaction(this, filebrowser_str);
 	urlview = new urlview_formaction(this, urlview_str);
 	selecttag = new select_formaction(this, selecttag_str);
-	searchresult = new itemlist_formaction(this, itemlist_str);
 
 	// push the dialog to start with onto the stack
 	formaction_stack.push_front(feedlist);
@@ -75,11 +76,21 @@ view::~view() {
 	delete filebrowser;
 	delete urlview;
 	delete selecttag;
-	delete searchresult;
 }
 
 void view::set_config_container(configcontainer * cfgcontainer) {
 	cfg = cfgcontainer;	
+	if (cfg) {
+		if (cfg->get_configvalue_as_bool("show-keymap-hint") == false) {
+			feedlist->get_form()->set("showhint", "0");
+			itemlist->get_form()->set("showhint", "0");
+			itemview->get_form()->set("showhint", "0");
+			helpview->get_form()->set("showhint", "0");
+			filebrowser->get_form()->set("showhint", "0");
+			urlview->get_form()->set("showhint", "0");
+			selecttag->get_form()->set("showhint", "0");
+		}
+	}
 }
 
 void view::set_keymap(keymap * k) {
@@ -90,17 +101,17 @@ void view::set_keymap(keymap * k) {
 
 
 void view::set_bindings() {
-	formaction * fas2bind[] = { feedlist, itemlist, itemview, helpview, filebrowser, urlview, selecttag, searchresult, NULL };
+	formaction * fas2bind[] = { feedlist, itemlist, itemview, helpview, filebrowser, urlview, selecttag, NULL };
 	if (keys) {
-		std::string upkey("** "); upkey.append(keys->getkey(OP_SK_UP));
-		std::string downkey("** "); downkey.append(keys->getkey(OP_SK_DOWN));
-		std::string pgupkey("** "); pgupkey.append(keys->getkey(OP_SK_PGUP));
-		std::string pgdownkey("** "); pgdownkey.append(keys->getkey(OP_SK_PGDOWN));
-
-		std::string pgupkey_itemview("** b "); pgupkey_itemview.append(keys->getkey(OP_SK_PGUP));
-		std::string pgdownkey_itemview("** SPACE "); pgdownkey_itemview.append(keys->getkey(OP_SK_PGDOWN));
-
 		for (unsigned int i=0;fas2bind[i];++i) {
+			std::string upkey("** "); upkey.append(keys->getkey(OP_SK_UP, fas2bind[i]->id()));
+			std::string downkey("** "); downkey.append(keys->getkey(OP_SK_DOWN, fas2bind[i]->id()));
+			std::string pgupkey("** "); pgupkey.append(keys->getkey(OP_SK_PGUP, fas2bind[i]->id()));
+			std::string pgdownkey("** "); pgdownkey.append(keys->getkey(OP_SK_PGDOWN, fas2bind[i]->id()));
+
+			std::string pgupkey_itemview("** b "); pgupkey_itemview.append(keys->getkey(OP_SK_PGUP, fas2bind[i]->id()));
+			std::string pgdownkey_itemview("** SPACE "); pgdownkey_itemview.append(keys->getkey(OP_SK_PGDOWN, fas2bind[i]->id()));
+
 			fas2bind[i]->get_form()->set("bind_up", upkey);
 			fas2bind[i]->get_form()->set("bind_down", downkey);
 			if (fas2bind[i] == itemview || fas2bind[i] == helpview) { // the forms that contain textviews
@@ -161,7 +172,11 @@ void view::run() {
 
 			// we then receive the event and ignore timeouts.
 			const char * event = fa->get_form()->run(0);
-			if (!event || strcmp(event,"TIMEOUT")==0) continue;
+			if (!event || strcmp(event,"TIMEOUT")==0) {
+				if (fa->id() == "article")
+					itemview->update_percent();
+				continue;
+			}
 
 			GetLogger().log(LOG_DEBUG, "view::run: event = %s", event);
 
@@ -174,7 +189,7 @@ void view::run() {
 				macrocmds = keys->get_macro(event);
 				set_status("");
 			} else {
-				op = keys->get_operation(event);
+				op = keys->get_operation(event, fa->id());
 
 				GetLogger().log(LOG_DEBUG, "view::run: event = %s op = %u", event, op);
 
@@ -218,7 +233,7 @@ std::string view::run_modal(formaction * f, const std::string& value) {
 		GetLogger().log(LOG_DEBUG, "view::run: event = %s", event);
 		if (!event || strcmp(event,"TIMEOUT")==0) continue;
 
-		operation op = keys->get_operation(event);
+		operation op = keys->get_operation(event, fa->id());
 
 		if (OP_REDRAW == op) {
 			stfl::reset();
@@ -254,25 +269,38 @@ std::string view::get_filename_suggestion(const std::string& s) {
 	return retval;	
 }
 
+void view::push_empty_formaction() {
+	formaction_stack.push_front(NULL);
+}
 
 void view::open_in_browser(const std::string& url) {
 	formaction_stack.push_front(NULL); // we don't want a thread to write over the browser
 	std::string cmdline;
 	std::string browser = cfg->get_configvalue("browser");
-	if (browser != "")
-		cmdline.append(browser);
-	else
-		cmdline.append("lynx");
-	cmdline.append(" '");
-	cmdline.append(url);
-	cmdline.append("'");
+	if (browser.find("%u") != std::string::npos) {
+		fmtstr_formatter fmt;
+		std::string newurl;
+		newurl = utils::replace_all(url, "'", "%27");
+		newurl.insert(0, "'");
+		newurl.append("'");
+		fmt.register_fmt('u', newurl);
+		cmdline = fmt.do_format(browser, 0);
+	} else {
+		if (browser != "")
+			cmdline.append(browser);
+		else
+			cmdline.append("lynx");
+		cmdline.append(" '");
+		cmdline.append(utils::replace_all(url,"'", "%27"));
+		cmdline.append("'");
+	}
 	stfl::reset();
 	GetLogger().log(LOG_DEBUG, "view::open_in_browser: running `%s'", cmdline.c_str());
 	::system(cmdline.c_str());
 	formaction_stack.pop_front();
 }
 
-void view::update_visible_feeds(std::vector<rss_feed>& feeds) {
+void view::update_visible_feeds(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds) {
 	scope_mutex lock(mtx);
 	try {
 		feedlist->update_visible_feeds(feeds);
@@ -282,11 +310,11 @@ void view::update_visible_feeds(std::vector<rss_feed>& feeds) {
 	}
 }
 
-void view::set_feedlist(std::vector<rss_feed>& feeds) {
+void view::set_feedlist(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds) {
 	scope_mutex lock(mtx);
 
-	for (std::vector<rss_feed>::iterator it=feeds.begin();it!=feeds.end();++it) {
-		if (it->rssurl().substr(0,6) != "query:") {
+	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it=feeds.begin();it!=feeds.end();++it) {
+		if ((*it)->rssurl().substr(0,6) != "query:") {
 			ctrl->set_feedptrs(*it);
 		}
 	}
@@ -304,21 +332,22 @@ void view::set_tags(const std::vector<std::string>& t) {
 	feedlist->set_tags(t);
 }
 
-void view::push_searchresult(rss_feed * feed) {
+void view::push_searchresult(std::tr1::shared_ptr<rss_feed> feed) {
 	assert(feed != NULL);
+	GetLogger().log(LOG_DEBUG, "view::push_searchresult: pushing search result");
 
 	if (feed->items().size() > 0) {
-		searchresult->set_feed(feed);
-		searchresult->set_show_searchresult(true);
-		searchresult->init();
-		formaction_stack.push_front(searchresult);
+		itemlist->set_feed(feed);
+		itemlist->set_show_searchresult(true);
+		itemlist->init();
+		formaction_stack.push_front(itemlist);
 	} else {
 		show_error(_("Error: feed contains no items!"));
 	}
 
 }
 
-void view::push_itemlist(rss_feed * feed) {
+void view::push_itemlist(std::tr1::shared_ptr<rss_feed> feed) {
 	assert(feed != NULL);
 
 	if (feed->rssurl().substr(0,6) == "query:") {
@@ -338,13 +367,13 @@ void view::push_itemlist(rss_feed * feed) {
 }
 
 void view::push_itemlist(unsigned int pos) {
-	rss_feed * feed = ctrl->get_feed(pos);
-	GetLogger().log(LOG_DEBUG, "view::push_itemlist: retrieved feed at position %d (address = %p)", pos, feed);
+	std::tr1::shared_ptr<rss_feed> feed = ctrl->get_feed(pos);
+	GetLogger().log(LOG_DEBUG, "view::push_itemlist: retrieved feed at position %d", pos);
 	itemlist->set_pos(pos);
 	push_itemlist(feed);
 }
 
-void view::push_itemview(rss_feed * f, const std::string& guid) {
+void view::push_itemview(std::tr1::shared_ptr<rss_feed> f, const std::string& guid) {
 	itemview->set_feed(f);
 	itemview->set_guid(guid);
 	itemview->init();
@@ -352,6 +381,7 @@ void view::push_itemview(rss_feed * f, const std::string& guid) {
 }
 
 void view::push_help() {
+	helpview->set_context((*formaction_stack.begin())->id());
 	formaction_stack.push_front(helpview);
 	helpview->init();
 }
@@ -408,10 +438,10 @@ char view::confirm(const std::string& prompt, const std::string& charset) {
 	return result;
 }
 
-void view::notify_itemlist_change(rss_feed& feed) {
-	rss_feed * f = itemlist->get_feed();
-	if (f != NULL && f->rssurl() == feed.rssurl()) {
-		itemlist->do_update_visible_items();
+void view::notify_itemlist_change(std::tr1::shared_ptr<rss_feed>& feed) {
+	std::tr1::shared_ptr<rss_feed> f = itemlist->get_feed();
+	if (f != NULL && f->rssurl() == feed->rssurl()) {
+		itemlist->set_feed(feed);
 		itemlist->set_redraw(true);
 	}
 }
@@ -425,6 +455,9 @@ bool view::get_previous_unread() {
 		itemview->set_feed(itemlist->get_feed());
 		itemview->set_guid(itemlist->get_guid());
 		return true;
+	} else if (cfg->get_configvalue_as_bool("goto-next-feed")==false) {
+		GetLogger().log(LOG_DEBUG, "view::get_previous_unread: goto-next-feed = false");
+		show_error(_("No unread items."));
 	} else if (feedlist->jump_to_previous_unread_feed(feedpos)) {
 		GetLogger().log(LOG_DEBUG, "view::get_previous_unread: found feed with unread articles");
 		itemlist->set_feed(feedlist->get_feed());
@@ -471,6 +504,9 @@ bool view::get_next_unread() {
 		itemview->set_feed(itemlist->get_feed());
 		itemview->set_guid(itemlist->get_guid());
 		return true;
+	} else if (cfg->get_configvalue_as_bool("goto-next-feed")==false) {
+		GetLogger().log(LOG_DEBUG, "view::get_next_unread: goto-next-feed = false");
+		show_error(_("No unread items."));
 	} else if (feedlist->jump_to_next_unread_feed(feedpos)) {
 		GetLogger().log(LOG_DEBUG, "view::get_next_unread: found feed with unread articles");
 		itemlist->set_feed(feedlist->get_feed());
@@ -499,12 +535,12 @@ void view::pop_current_formaction() {
 }
 
 
-void view::set_colors(const colormanager& colorman) {
-	std::map<std::string,std::string>::const_iterator fgcit = colorman.fg_colors.begin();
-	std::map<std::string,std::string>::const_iterator bgcit = colorman.bg_colors.begin();
-	std::map<std::string,std::vector<std::string> >::const_iterator attit = colorman.attributes.begin();
+void view::set_colors(std::map<std::string,std::string>& fg_colors, std::map<std::string,std::string>& bg_colors, std::map<std::string,std::vector<std::string> >& attributes) {
+	std::map<std::string,std::string>::const_iterator fgcit = fg_colors.begin();
+	std::map<std::string,std::string>::const_iterator bgcit = bg_colors.begin();
+	std::map<std::string,std::vector<std::string> >::const_iterator attit = attributes.begin();
 
-	for (;fgcit != colorman.fg_colors.end(); ++fgcit, ++bgcit, ++attit) {
+	for (;fgcit != fg_colors.end(); ++fgcit, ++bgcit, ++attit) {
 		std::string colorattr;
 		if (fgcit->second != "default") {
 			colorattr.append("fg=");
@@ -532,7 +568,6 @@ void view::set_colors(const colormanager& colorman) {
 		filebrowser->get_form()->set(fgcit->first, colorattr);
 		urlview->get_form()->set(fgcit->first, colorattr);
 		selecttag->get_form()->set(fgcit->first, colorattr);
-		searchresult->get_form()->set(fgcit->first, colorattr);
 
 		if (fgcit->first == "article") {
 			std::string styleend_str;
@@ -569,5 +604,11 @@ void view::feedlist_mark_pos_if_visible(unsigned int pos) {
 	feedlist->mark_pos_if_visible(pos);
 }
 
+void view::set_regexmanager(regexmanager * r) {
+	itemview->set_regexmanager(r);
+	itemlist->set_regexmanager(r);
+	feedlist->set_regexmanager(r);
+}
+
 
 }
