@@ -3,6 +3,8 @@
 #define BOOST_AUTO_TEST_MAIN
 #define BOOST_TEST_DYN_LINK
 #include <climits>
+#include <vector>
+#include <string>
 #include <boost/test/auto_unit_test.hpp>
 
 #include <unistd.h>
@@ -10,6 +12,7 @@
 #include <logger.h>
 #include <cache.h>
 #include <rss.h>
+#include <rss_parser.h>
 #include <configcontainer.h>
 #include <keymap.h>
 #include <tagsouppullparser.h>
@@ -19,6 +22,8 @@
 #include <history.h>
 #include <formatstring.h>
 #include <exceptions.h>
+#include <regexmanager.h>
+#include <htmlrenderer.h>
 
 #include <stdlib.h>
 
@@ -36,29 +41,29 @@ BOOST_AUTO_TEST_CASE(TestNewsbeuterReload) {
 	cache * rsscache = new cache("test-cache.db", cfg);
 
 	rss_parser parser("http://bereshit.synflood.at/~ak/rss.xml", rsscache, cfg, NULL);
-	rss_feed feed = parser.parse();
-	BOOST_CHECK_EQUAL(feed.items().size(), 8u);
+	std::tr1::shared_ptr<rss_feed> feed = parser.parse();
+	BOOST_CHECK_EQUAL(feed->items().size(), 8u);
 
-	rsscache->externalize_rssfeed(feed);
+	rsscache->externalize_rssfeed(feed, false);
 	rsscache->internalize_rssfeed(feed);
-	BOOST_CHECK_EQUAL(feed.items().size(), 8u);
+	BOOST_CHECK_EQUAL(feed->items().size(), 8u);
 
-	BOOST_CHECK_EQUAL(feed.items()[0].title(), "Teh Saxxi");
-	BOOST_CHECK_EQUAL(feed.items()[7].title(), "Handy als IR-Detektor");
+	BOOST_CHECK_EQUAL(feed->items()[0]->title(), "Teh Saxxi");
+	BOOST_CHECK_EQUAL(feed->items()[7]->title(), "Handy als IR-Detektor");
 
-	feed.items()[0].set_title("Another Title");
-	feed.items()[0].set_pubDate(time(NULL));
-	BOOST_CHECK_EQUAL(feed.items()[0].title(), "Another Title");
+	feed->items()[0]->set_title("Another Title");
+	feed->items()[0]->set_pubDate(time(NULL));
+	BOOST_CHECK_EQUAL(feed->items()[0]->title(), "Another Title");
 
-	rsscache->externalize_rssfeed(feed);
+	rsscache->externalize_rssfeed(feed, false);
 
-	rss_feed feed2(rsscache);
-	feed2.set_rssurl("http://bereshit.synflood.at/~ak/rss.xml");
+	std::tr1::shared_ptr<rss_feed> feed2(new rss_feed(rsscache));
+	feed2->set_rssurl("http://bereshit.synflood.at/~ak/rss.xml");
 	rsscache->internalize_rssfeed(feed2);
 
-	BOOST_CHECK_EQUAL(feed2.items().size(), 8u);
-	BOOST_CHECK_EQUAL(feed2.items()[0].title(), "Another Title");
-	BOOST_CHECK_EQUAL(feed2.items()[7].title(), "Handy als IR-Detektor");
+	BOOST_CHECK_EQUAL(feed2->items().size(), 8u);
+	BOOST_CHECK_EQUAL(feed2->items()[0]->title(), "Another Title");
+	BOOST_CHECK_EQUAL(feed2->items()[7]->title(), "Handy als IR-Detektor");
 
 	rsscache->set_lastmodified("http://bereshit.synflood.at/~ak/rss.xml", 1000);
 	BOOST_CHECK_EQUAL(rsscache->get_lastmodified("http://bereshit.synflood.at/~ak/rss.xml"), 1000);
@@ -69,13 +74,12 @@ BOOST_AUTO_TEST_CASE(TestNewsbeuterReload) {
 	BOOST_CHECK_EQUAL(feedurls.size(), 1u);
 	BOOST_CHECK_EQUAL(feedurls[0], "http://bereshit.synflood.at/~ak/rss.xml");
 
-	std::vector<rss_feed> feedv;
+	std::vector<std::tr1::shared_ptr<rss_feed> > feedv;
 	feedv.push_back(feed);
 
 	cfg->set_configvalue("cleanup-on-quit", "true");
 	rsscache->cleanup_cache(feedv);
 
-
 	delete rsscache;
 	delete cfg;
 
@@ -116,22 +120,22 @@ BOOST_AUTO_TEST_CASE(TestConfigParserContainerAndKeymap) {
 
 	delete cfg;
 
-	BOOST_CHECK_EQUAL(k.get_operation("ENTER"), OP_OPEN);
-	BOOST_CHECK_EQUAL(k.get_operation("u"), OP_SHOWURLS);
-	BOOST_CHECK_EQUAL(k.get_operation("X"), OP_NIL);
-	BOOST_CHECK_EQUAL(k.get_operation(""), OP_NIL);
+	BOOST_CHECK_EQUAL(k.get_operation("ENTER", "feedlist"), OP_OPEN);
+	BOOST_CHECK_EQUAL(k.get_operation("u", "article"), OP_SHOWURLS);
+	BOOST_CHECK_EQUAL(k.get_operation("X", "feedlist"), OP_NIL);
+	BOOST_CHECK_EQUAL(k.get_operation("", "feedlist"), OP_NIL);
 
-	k.unset_key("ENTER");
-	BOOST_CHECK_EQUAL(k.get_operation("ENTER"), OP_NIL);
-	k.set_key(OP_OPEN, "ENTER");
-	BOOST_CHECK_EQUAL(k.get_operation("ENTER"), OP_OPEN);
+	k.unset_key("ENTER", "all");
+	BOOST_CHECK_EQUAL(k.get_operation("ENTER", "all"), OP_NIL);
+	k.set_key(OP_OPEN, "ENTER", "all");
+	BOOST_CHECK_EQUAL(k.get_operation("ENTER", "all"), OP_OPEN);
 
 	BOOST_CHECK_EQUAL(k.get_opcode("open"), OP_OPEN);
 	BOOST_CHECK_EQUAL(k.get_opcode("some-noexistent-operation"), OP_NIL);
 
-	BOOST_CHECK_EQUAL(k.getkey(OP_OPEN), "ENTER");
-	BOOST_CHECK_EQUAL(k.getkey(OP_TOGGLEITEMREAD), "N");
-	BOOST_CHECK_EQUAL(k.getkey(static_cast<operation>(30000)), "<none>");
+	BOOST_CHECK_EQUAL(k.getkey(OP_OPEN, "all"), "ENTER");
+	BOOST_CHECK_EQUAL(k.getkey(OP_TOGGLEITEMREAD, "all"), "N");
+	BOOST_CHECK_EQUAL(k.getkey(static_cast<operation>(30000), "all"), "<none>");
 
 	BOOST_CHECK_EQUAL(k.get_key(" "), ' ');
 	BOOST_CHECK_EQUAL(k.get_key("U"), 'U');
@@ -164,21 +168,17 @@ BOOST_AUTO_TEST_CASE(TestTagSoupPullParser) {
 	e = xpp.next();
 	BOOST_CHECK_EQUAL(e, tagsouppullparser::START_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "test");
-	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), 0);
 	e = xpp.next();
 	BOOST_CHECK_EQUAL(e, tagsouppullparser::START_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "foo");
-	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), 2);
 	BOOST_CHECK_EQUAL(xpp.getAttributeValue("quux"), "asdf");
 	BOOST_CHECK_EQUAL(xpp.getAttributeValue("bar"), "qqq");
 	e = xpp.next();
 	BOOST_CHECK_EQUAL(e, tagsouppullparser::TEXT);
 	BOOST_CHECK_EQUAL(xpp.getText(), "text");
-	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), -1);
 	e = xpp.next();
 	BOOST_CHECK_EQUAL(e, tagsouppullparser::END_TAG);
 	BOOST_CHECK_EQUAL(xpp.getText(), "foo");
-	BOOST_CHECK_EQUAL(xpp.getAttributeCount(), -1);
 	e = xpp.next();
 	BOOST_CHECK_EQUAL(e, tagsouppullparser::TEXT);
 	BOOST_CHECK_EQUAL(xpp.getText(), "more text");
@@ -370,6 +370,17 @@ BOOST_AUTO_TEST_CASE(TestFilterLanguage) {
 	BOOST_CHECK_EQUAL(m.matches(&tm), false);
 	m.parse("tags # \"foo\" or tags # \"xyz\"");
 	BOOST_CHECK_EQUAL(m.matches(&tm), true);
+
+	m.parse("AAAA > 12344");
+	BOOST_CHECK_EQUAL(m.matches(&tm), true);
+	m.parse("AAAA > 12345");
+	BOOST_CHECK_EQUAL(m.matches(&tm), false);
+	m.parse("AAAA >= 12345");
+	BOOST_CHECK_EQUAL(m.matches(&tm), true);
+	m.parse("AAAA < 12345");
+	BOOST_CHECK_EQUAL(m.matches(&tm), false);
+	m.parse("AAAA <= 12345");
+	BOOST_CHECK_EQUAL(m.matches(&tm), true);
 }
 
 BOOST_AUTO_TEST_CASE(TestFilterLanguageMemMgmt) {
@@ -466,8 +477,15 @@ BOOST_AUTO_TEST_CASE(TestMiscUtilsFunctions) {
 
 	BOOST_CHECK_EQUAL(utils::get_command_output("ls /dev/null"), "/dev/null\n");
 
-	BOOST_CHECK_EQUAL(utils::run_filter("cat", "this is a multine-line\ntest string"), "this is a multine-line\ntest string");
-	BOOST_CHECK_EQUAL(utils::run_filter("echo -n 'hello world'", ""), "hello world");
+	char * argv[4];
+	argv[0] = "cat";
+	argv[1] = NULL;
+	BOOST_CHECK_EQUAL(utils::run_program(argv, "this is a multine-line\ntest string"), "this is a multine-line\ntest string");
+	argv[0] = "echo";
+	argv[1] = "-n";
+	argv[2] = "hello world";
+	argv[3] = NULL;
+	BOOST_CHECK_EQUAL(utils::run_program(argv, ""), "hello world");
 
 	BOOST_CHECK_EQUAL(utils::replace_all("aaa", "a", "b"), "bbb");
 	BOOST_CHECK_EQUAL(utils::replace_all("aaa", "aa", "ba"), "baa");
@@ -493,4 +511,125 @@ BOOST_AUTO_TEST_CASE(TestUtilsStrPrintf) {
 	BOOST_CHECK_EQUAL(utils::strprintf("%u-%s-%c", 23, "hello world", 'X'), "23-hello world-X");
 }
 
+BOOST_AUTO_TEST_CASE(TestRegexManager) {
 
+	regexmanager rxman;
+
+	std::vector<std::string> params;
+	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_TOO_FEW_PARAMS);
+	params.push_back("articlelist");
+	params.push_back("foo");
+	params.push_back("blue");
+	params.push_back("red");
+
+	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+
+	std::string str = "xfoox";
+	rxman.quote_and_highlight(str, "articlelist");
+	BOOST_CHECK_EQUAL(str, "x<0>foo</>x");
+
+	str = "xfoox";
+	rxman.quote_and_highlight(str, "feedlist");
+	BOOST_CHECK_EQUAL(str, "xfoox");
+
+	params[0] = "feedlist";
+	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+
+	str = "yfooy";
+	rxman.quote_and_highlight(str, "feedlist");
+	BOOST_CHECK_EQUAL(str, "y<0>foo</>y");
+
+	params[0] = "invalidloc";
+	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_INVALID_PARAMS);
+
+	params[0] = "feedlist";
+	params[1] = "*";
+	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_INVALID_PARAMS);
+
+	params[1] = "foo";
+	params.push_back("bold");
+	params.push_back("underline");
+	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+
+	params[0] = "all";
+	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+
+	BOOST_CHECK_EQUAL(rxman.handle_action("invalidcommand", params), AHS_INVALID_COMMAND);
+
+	str = "<";
+	rxman.quote_and_highlight(str, "feedlist");
+	BOOST_CHECK_EQUAL(str, "<>");
+
+	str = "a<b>";
+	rxman.quote_and_highlight(str, "feedlist");
+	BOOST_CHECK_EQUAL(str, "a<>b>");
+}
+BOOST_AUTO_TEST_CASE(TestHtmlRenderer) {
+	htmlrenderer rnd(100);
+
+	std::vector<std::string> lines;
+	std::vector<linkpair> links;
+
+	rnd.render("<a href=\"http://slashdot.org/\">slashdot</a>", lines, links, "");
+	BOOST_CHECK(lines.size() >= 1);
+	BOOST_CHECK_EQUAL(lines[0], "[slashdot][1]");
+	BOOST_CHECK_EQUAL(links[0].first, "http://slashdot.org/");
+	BOOST_CHECK_EQUAL(links[0].second, LINK_HREF);
+
+	lines.erase(lines.begin(), lines.end());
+	links.erase(links.begin(), links.end());
+
+	rnd.render("hello<br />world!", lines, links, "");
+	BOOST_CHECK(lines.size() >= 2);
+	BOOST_CHECK_EQUAL(lines[0], "hello");
+	BOOST_CHECK_EQUAL(lines[1], "world!");
+
+	lines.erase(lines.begin(), lines.end());
+	links.erase(links.begin(), links.end());
+
+	rnd.render("3<sup>10</sup>", lines, links, "");
+	BOOST_CHECK(lines.size() >= 1);
+	BOOST_CHECK_EQUAL(lines[0], "3^10");
+
+	lines.erase(lines.begin(), lines.end());
+	links.erase(links.begin(), links.end());
+
+	rnd.render("A<sub>i</sub>", lines, links, "");
+	BOOST_CHECK(lines.size() >= 1);
+	BOOST_CHECK_EQUAL(lines[0], "A[i]");
+}
+
+BOOST_AUTO_TEST_CASE(TestIndexPartitioning) {
+	std::vector<std::pair<unsigned int, unsigned int> > partitions = utils::partition_indexes(0, 9, 2);
+	BOOST_CHECK_EQUAL(partitions.size(), 2u);
+	BOOST_CHECK_EQUAL(partitions[0].first, 0u);
+	BOOST_CHECK_EQUAL(partitions[0].second, 4u);
+	BOOST_CHECK_EQUAL(partitions[1].first, 5u);
+	BOOST_CHECK_EQUAL(partitions[1].second, 9u);
+
+	partitions = utils::partition_indexes(0, 10, 3);
+	BOOST_CHECK_EQUAL(partitions.size(), 3u);
+	BOOST_CHECK_EQUAL(partitions[0].first, 0u);
+	BOOST_CHECK_EQUAL(partitions[0].second, 2u);
+	BOOST_CHECK_EQUAL(partitions[1].first, 3u);
+	BOOST_CHECK_EQUAL(partitions[1].second, 5u);
+	BOOST_CHECK_EQUAL(partitions[2].first, 6u);
+	BOOST_CHECK_EQUAL(partitions[2].second, 10u);
+
+	partitions = utils::partition_indexes(0, 11, 3);
+	BOOST_CHECK_EQUAL(partitions.size(), 3u);
+	BOOST_CHECK_EQUAL(partitions[0].first, 0u);
+	BOOST_CHECK_EQUAL(partitions[0].second, 3u);
+	BOOST_CHECK_EQUAL(partitions[1].first, 4u);
+	BOOST_CHECK_EQUAL(partitions[1].second, 7u);
+	BOOST_CHECK_EQUAL(partitions[2].first, 8u);
+	BOOST_CHECK_EQUAL(partitions[2].second, 11u);
+
+	partitions = utils::partition_indexes(0, 199, 200);
+	BOOST_CHECK_EQUAL(partitions.size(), 200u);
+
+	partitions = utils::partition_indexes(0, 103, 1);
+	BOOST_CHECK_EQUAL(partitions.size(), 1u);
+	BOOST_CHECK_EQUAL(partitions[0].first, 0u);
+	BOOST_CHECK_EQUAL(partitions[0].second, 103u);
+}
