@@ -52,6 +52,7 @@ __nxml_parse_string (nxml_t * doc, char *buffer, size_t size)
       if (*(buffer + i) == 0xd)
 	continue;
 
+	/*
       if (*(buffer + i) == 0xa || *(buffer + i) == 0x9
 	  || *(buffer + i) == 0x20)
 	{
@@ -64,7 +65,7 @@ __nxml_parse_string (nxml_t * doc, char *buffer, size_t size)
 	    }
 	}
 
-      else if (*(buffer + i) == '&')
+      else */ if (*(buffer + i) == '&')
 	{
 	  if (!strncmp (buffer + i, "&lt;", 4))
 	    {
@@ -1325,6 +1326,9 @@ __nxml_parse_buffer (nxml_t * nxml, const char *r_buffer, size_t r_size)
       size -= 2;
     }
 
+  if (!nxml->encoding)
+    nxml->encoding = strdup("utf-8");
+
   root = last = NULL;
   while (!(err = __nxml_parse_get_tag (nxml, &buffer, &size, &tag, &doctype))
 	 && (doctype || tag))
